@@ -59,6 +59,15 @@
 #'   should be enabled when removing the intercept by adding \code{0} 
 #'   to the right-hand of model formulas. Defaults to \code{TRUE} to 
 #'   mirror the behavior of standard \R formula parsing.
+#' @param sparse Logical; indicates whether the population-level design matrices
+#'   should be treated as sparse (defaults to \code{FALSE}). For design matrices
+#'   with many zeros, this can considerably reduce required memory. Sampling
+#'   speed is currently not improved or even slightly decreased.
+#' @param decomp Optional name of the decomposition used for the 
+#'   population-level design matrix. Defaults to \code{NULL} that is
+#'   no decomposition. Other options currently available are
+#'   \code{"QR"} for the QR decomposition that helps in fitting models
+#'   with highly correlated predictors.
 #' @param family Same argument as in \code{\link{brm}}.
 #'   If \code{family} is specified in \code{brmsformula}, it will 
 #'   overwrite the value specified in \code{\link{brm}}.
@@ -574,7 +583,8 @@
 #' @export
 brmsformula <- function(formula, ..., flist = NULL, family = NULL,
                         autocor = NULL, nl = NULL, loop = NULL, 
-                        center = NULL, cmc = NULL) {
+                        center = NULL, cmc = NULL, sparse = NULL,
+                        decomp = NULL) {
   if (is.brmsformula(formula)) {
     out <- formula
   } else {
@@ -587,7 +597,7 @@ brmsformula <- function(formula, ..., flist = NULL, family = NULL,
   dots <- unlist(dots, recursive = FALSE)
   forms <- list()
   for (i in seq_along(dots)) {
-    forms <- c(forms, prepare_auxformula(dots[[i]], par = names(dots)[i]))
+    c(forms) <- validate_par_formula(dots[[i]], par = names(dots)[i])
   }
   is_dupl_pars <- duplicated(names(forms), fromLast = TRUE)
   if (any(is_dupl_pars)) {
@@ -652,16 +662,20 @@ brmsformula <- function(formula, ..., flist = NULL, family = NULL,
   if (!is.null(cmc)) {
     attr(out$formula, "cmc") <- as_one_logical(cmc)
   }
+  if (!is.null(sparse)) {
+    attr(out$formula, "sparse") <- as_one_logical(sparse)
+  }
+  if (!is.null(decomp)) {
+    attr(out$formula, "decomp") <- match.arg(decomp, decomp_opts())
+  }
   if (!is.null(family)) {
     out$family <- check_family(family)
   }
   if (!is.null(autocor)) {
     out$autocor <- check_autocor(autocor)
   }
-  respform <- lhs(formula)
-  if (!is.null(respform)) {
-    respform <- formula(gsub("\\|+[^~]*~", "~", formula2str(respform)))
-    out$resp <- parse_resp(respform)
+  if (!is.null(lhs(formula))) {
+    out$resp <- parse_resp(formula)
   }
   # add default values for unspecified elements
   defs <- list(
@@ -674,13 +688,15 @@ brmsformula <- function(formula, ..., flist = NULL, family = NULL,
   split_bf(out)
 }
 
+# alias of brmsformula
 #' @export
 bf <- function(formula, ..., flist = NULL, family = NULL, autocor = NULL,
-               nl = NULL, loop = NULL, center = NULL, cmc = NULL) {
-  # alias of brmsformula
+               nl = NULL, loop = NULL, center = NULL, cmc = NULL,
+               sparse = NULL, decomp = NULL) {
   brmsformula(
     formula, ..., flist = flist, family = family, autocor = autocor, 
-    nl = nl, loop = loop, center = center, cmc = cmc
+    nl = nl, loop = loop, center = center, cmc = cmc, sparse = sparse,
+    decomp = decomp
   )
 }
 
@@ -769,7 +785,8 @@ nlf <- function(formula, ..., flist = NULL, dpar = NULL,
 #' @rdname brmsformula-helpers
 #' @export
 lf <- function(..., flist = NULL, dpar = NULL, resp = NULL, 
-               center = NULL, cmc = NULL) {
+               center = NULL, cmc = NULL, sparse = NULL, 
+               decomp = NULL) {
   out <- c(list(...), flist)
   warn_dpar(dpar)
   if (!is.null(resp)) {
@@ -777,6 +794,7 @@ lf <- function(..., flist = NULL, dpar = NULL, resp = NULL,
   }
   cmc <- if (!is.null(cmc)) as_one_logical(cmc)
   center <- if (!is.null(center)) as_one_logical(center)
+  decomp <- if (!is.null(decomp)) match.arg(decomp, decomp_opts())
   for (i in seq_along(out)) {
     if (!is.null(cmc)) {
       attr(out[[i]], "cmc") <- cmc
@@ -784,6 +802,12 @@ lf <- function(..., flist = NULL, dpar = NULL, resp = NULL,
     if (!is.null(center)) {
       attr(out[[i]], "center") <- center
     }
+    if (!is.null(sparse)) {
+      attr(out[[i]], "sparse") <- sparse
+    }
+    if (!is.null(decomp)) {
+      attr(out[[i]], "decomp") <- decomp
+    }
   }
   structure(out, resp = resp)
 }
@@ -869,14 +893,13 @@ mvbf <- function(..., flist = NULL, rescor = NULL) {
   mvbrmsformula(..., flist = flist, rescor = rescor)
 }
 
+# build a mvbrmsformula object based on a brmsformula object
+# which uses mvbind on the left-hand side to specify MV models
 split_bf <- function(x) {
-  # build a mvbrmsformula object based on a brmsformula object
-  # which uses mvbind on the left-hand side to specify MV models
   stopifnot(is.brmsformula(x))
   resp <- parse_resp(x$formula, check_names = FALSE)
-  str_adform <- get_matches(
-    "\\|[^~]*(?=~)", formula2str(x$formula), perl = TRUE
-  )
+  str_adform <- formula2str(x$formula)
+  str_adform <- get_matches("\\|[^~]*(?=~)", str_adform, perl = TRUE)
   if (length(resp) > 1L) {
     # mvbind syntax used to specify MV model
     flist <- named_list(resp)
@@ -921,8 +944,10 @@ allow_rescor <- function(x) {
     return(FALSE)
   }
   parts <- if (is.mvbrmsformula(x)) x$forms else x$terms 
-  families <- ulapply(parts, function(f) f$family$family)
-  all(families == "gaussian") || all(families == "student")
+  families <- lapply(parts, "[[", "family")
+  has_rescor <- ulapply(families, has_rescor)
+  family_names <- ulapply(families, "[[", "family")
+  all(has_rescor) && length(unique(family_names)) == 1L
 }
 
 #' @rdname brmsformula-helpers
@@ -942,7 +967,8 @@ set_mecor <- function(mecor = TRUE) {
   }
   out
 }
-  
+
+# internal helper function of '+.bform'
 plus_brmsformula <- function(e1, e2) {
   if (is.function(e2)) {
     e2 <- try(e2(), silent = TRUE)
@@ -977,6 +1003,7 @@ plus_brmsformula <- function(e1, e2) {
   e1
 }
 
+# internal helper function of '+.bform'
 plus_mvbrmsformula <- function(e1, e2) {
   if (is.function(e2)) {
     e2 <- try(e2(), silent = TRUE)
@@ -1009,15 +1036,14 @@ plus_mvbrmsformula <- function(e1, e2) {
   e1
 }
 
+# extract the 'nl' attribute from a brmsformula object
+# @param x object to extract 'nl' from
+# @param dpar optional name of a distributional parameter
+#   for which 'nl' should be extracted
+# @param resp: optional name of a response variable
+#   for which 'nl' should be extracted
+# @param aol: (as one logical) apply isTRUE to the result?
 get_nl <- function(x, dpar = NULL, resp = NULL, aol = TRUE) {
-  # extract the 'nl' attribute from a (brms)formula object
-  # Args:
-  #   x: object to extract 'nl' from
-  #   dpar: optional name of a distributional parameter
-  #     for which 'nl' should be extracted
-  #   resp: optional name of a response variable
-  #     for which 'nl' should be extracted
-  #   aol: (as one logical) apply isTRUE to the result?
   if (is.mvbrmsformula(x)) {
     resp <- as_one_character(resp)
     x <- x$forms[[resp]]
@@ -1037,13 +1063,18 @@ get_nl <- function(x, dpar = NULL, resp = NULL, aol = TRUE) {
   nl
 }
 
-prepare_auxformula <- function(formula, par = NULL, rsv_pars = NULL) {
-  # validate and prepare a formula of an distributional parameter
-  # Args:
-  #   formula: an object of class formula
-  #   par: optional name of the parameter; if not specified
-  #        the parameter name will be inferred from the formula
-  #   rsv_pars: optional character vector of reserved parameter names
+# available options for the 'decomp' argument
+decomp_opts <- function() {
+  c("none", "QR")
+}
+
+# validate a formula of an additional parameter
+# @param formula an formula object
+# @param par optional name of the parameter; if not specified
+#   the parameter name will be inferred from the formula
+# @param rsv_pars optional character vector of reserved parameter names
+# @return a named list of length one containing the formula
+validate_par_formula <- function(formula, par = NULL, rsv_pars = NULL) {
   stopifnot(length(par) <= 1L)
   try_formula <- try(as.formula(formula), silent = TRUE)
   if (is(try_formula, "try-error")) {
@@ -1085,8 +1116,32 @@ prepare_auxformula <- function(formula, par = NULL, rsv_pars = NULL) {
   out
 }
 
+# validate formulas dedicated to response variables
+# @param x coerced to a formula object
+# @param empty_ok is an empty left-hand-side ok?
+# @return a formula of the form <response> ~ 1
+validate_resp_formula <- function(x, empty_ok = TRUE) {
+  out <- lhs(as.formula(x))
+  if (is.null(out)) {
+    if (empty_ok) {
+      out <- ~ 1
+    } else {
+      str_x <- formula2str(x, space = "trim")
+      stop2("Response variable is missing in formula ", str_x) 
+    }
+  }
+  out <- gsub("\\|+[^~]*~", "~", formula2str(out))
+  out <- try(formula(out), silent = TRUE)
+  if (is(out, "try-error")) {
+    str_x <- formula2str(x, space = "trim")
+    stop2("Incorrect use of '|' on the left-hand side of ", str_x)
+  }
+  environment(out) <- environment(x)
+  out
+}
+
+# incorporate additional arguments into the model formula
 validate_formula <- function(formula, ...) {
-  # incorporate additional arguments into the model formula
   UseMethod("validate_formula")
 }
 
@@ -1095,20 +1150,18 @@ validate_formula.default <- function(formula, ...) {
   validate_formula(bf(formula), ...)
 }
 
+# incorporate additional arguments into the model formula
+# @param formula object of class 'formula' of 'brmsformula'
+# @param data optional data.frame to validate data related arguments
+# @param family optional 'family' object
+# @param autocor optional 'cor_brms' object
+# @param threshold (deprecated) threshold type for ordinal models
+# @return a brmsformula object compatible with the current version of brms
 #' @export
 validate_formula.brmsformula <- function(
   formula, family = gaussian(), autocor = cor_empty(), 
-  data = NULL, threshold = NULL, ...
+  data = NULL, threshold = NULL, sparse = NULL, ...
 ) {
-  # incorporate additional arguments into the model formula
-  # Args:
-  #   formula: object of class 'formula' of 'brmsformula'
-  #   data: optional data.frame
-  #   family: optional object of class 'family'
-  #   autocor: optional object of class 'cor_brms'
-  #   threshold: (deprecated); threshold type for ordinal models
-  # Returns:
-  #   a brmsformula object compatible with the current version of brms
   out <- bf(formula)
   if (is.null(out$family) && !is.null(family)) {
     out$family <- check_family(family)
@@ -1121,10 +1174,30 @@ validate_formula.brmsformula <- function(
   for (i in seq_along(out$pforms)) {
     out$pforms[[i]] <- expand_dot_formula(out$pforms[[i]], data)
   }
+  if (!is.null(sparse)) {
+    # a global 'sparse' argument is deprecated as of brms 2.8.3
+    warning2(
+      "Argument 'sparse' should be specified within the ", 
+      "'formula' argument. See ?brmsformula for help."
+    )
+    sparse <- as_one_logical(sparse)
+    if (is.null(attr(out$formula, "sparse"))) {
+      attr(out$formula, "sparse") <- sparse
+    }
+    for (i in seq_along(out$pforms)) {
+      if (is.null(attr(out$pforms[[i]], "sparse"))) {
+        attr(out$pforms[[i]], "sparse") <- sparse
+      }
+    }
+  }
   out$mecor <- default_mecor(out$mecor)
+  if (has_cat(out) && is.null(get_cats(out)) && !is.null(data)) {
+    # for easy access of response categories
+    out$family$cats <- extract_cat_names(out, data)
+  }
   if (is_ordinal(out$family)) {
     if (is.null(out$family$threshold) && !is.null(threshold)) {
-      # slot 'threshold' is deprecated as of brms > 1.7.0
+      # slot 'threshold' is deprecated as of brms 1.7.0
       out$family <- check_family(out$family, threshold = threshold)
     }
     try_terms <- try(stats::terms(out$formula), silent = TRUE)
@@ -1132,11 +1205,17 @@ validate_formula.brmsformula <- function(
     if (!is(try_terms, "try-error") && isTRUE(intercept == 0)) {
       stop2("Cannot remove the intercept in an ordinal model.")
     }
+    if (is.mixfamily(out$family)) {
+      # every mixture family needs to know about response categories
+      for (i in seq_along(out$family$mix)) {
+        out$family$mix[[i]]$cats <- out$family$cats
+      }
+    }
   }
   mu_dpars <- str_subset(out$family$dpars, "^mu")
   conv_cats_dpars <- conv_cats_dpars(out$family)
   if (conv_cats_dpars && !length(mu_dpars) && !is.null(data)) {
-    out$family$cats <- extract_cat_names(out, data)
+    # define distributional parameters based on response categories
     if (length(out$family$cats) < 2L) {
       stop2("At least 2 response categories are required.")
     }
@@ -1164,12 +1243,12 @@ validate_formula.brmsformula <- function(
   bf(out)
 }
 
+# incorporate additional arguments into MV model formulas
+# allow passing lists of families or autocors
 #' @export
 validate_formula.mvbrmsformula <- function(
   formula, family = NULL, autocor = NULL, ...
 ) {
-  # incorporate additional arguments into the MV model formula
-  # allow passing lists of families or autocors
   nresp <- length(formula$forms)
   if (!is(family, "list")) {
     family <- replicate(nresp, family, simplify = FALSE)
@@ -1216,21 +1295,20 @@ validate_formula.mvbrmsformula <- function(
   formula
 }
 
+# update a brmsformula and / or its attributes
+# @param brmsformula object 
+# @param formula.: formula to update 'object'
+# @param mode supports the following options:
+#   "update": apply update.formula
+#   "replace": replace old formula
+#   "keep": keep old formula
+#   attributes are always updated
+# @param ... currently unused
+# @return a brmsformula object
 #' @export
 update.brmsformula <- function(object, formula., 
                                mode = c("update", "replace", "keep"), 
                                ...) {
-  # update a brmsformula and / or its attributes
-  # Args:
-  #   object: an object of class 'brmsformula'
-  #   formula.: formula to update object
-  #   mode: "update": apply update.formula
-  #         "replace": replace old formula
-  #         "keep": keep old formula
-  #         attributes are always updated
-  #   ...: currently unused
-  # Returns:
-  #   a brmsformula object
   mode <- match.arg(mode)
   object <- bf(object)
   up_family <- formula.[["family"]]
@@ -1379,7 +1457,7 @@ is_nonlinear <- function(x) {
 }
 
 warn_dpar <- function(dpar) {
-  # deprecated as of version 2.3.7
+  # argument 'dpar' in formula helper functions is deprecated as of 2.3.7
   if (!is.null(dpar)) {
     warning2("Argument 'dpar' is no longer necessary and ignored.")
   }
