@@ -47,21 +47,20 @@ data_predictor.brmsterms <- function(x, data, prior, ranef, knots = NULL,
   out
 }
 
+# prepare data for all types of effects for use in Stan
+# @param data the data passed by the user
+# @param ranef object retuend by 'tidy_ranef'
+# @param prior an object of class brmsprior
+# @param knots optional knot values for smoothing terms
+# @param not4stan is the data for use in S3 methods only?
+# @param old_sdata see 'extract_old_standata'
+# @param ... currently ignored
+# @return a named list of data to be passed to Stan
 #' @export
 data_predictor.btl <- function(x, data, ranef = empty_ranef(), 
                                prior = brmsprior(), knots = NULL, 
                                not4stan = FALSE, old_sdata = NULL, ...) {
-  # prepare data for all types of effects for use in Stan
-  # Args:
-  #   data: the data passed by the user
-  #   ranef: object retuend by 'tidy_ranef'
-  #   prior: an object of class brmsprior
-  #   knots: optional knot values for smoothing terms
-  #   not4stan: is the data for use in S3 methods only?
-  #   old_sdata: see 'extract_old_standata'
-  # Returns:
-  #   A named list of data to be passed to Stan
-  c(data_fe(x, data, not4stan = not4stan),
+  c(data_fe(x, data, not4stan = not4stan, ...),
     data_sp(x, data, prior = prior, Jmo = old_sdata$Jmo),
     data_re(x, data, ranef = ranef),
     data_cs(x, data),
@@ -72,10 +71,9 @@ data_predictor.btl <- function(x, data, ranef = empty_ranef(),
   )
 }
 
+# prepare data for non-linear parameters for use in Stan
 #' @export 
 data_predictor.btnl <- function(x, data, not4stan = FALSE, ...) {
-  # prepare data for non-linear parameters for use in Stan
-  # matrix of covariates appearing in the non-linear formula
   out <- list()
   C <- get_model_matrix(x$covars, data = data)
   if (length(all.vars(x$covars)) != ncol(C)) {
@@ -96,16 +94,12 @@ data_predictor.btnl <- function(x, data, not4stan = FALSE, ...) {
   out
 }
 
+# prepare data of fixed effects
 data_fe <- function(bterms, data, not4stan = FALSE) {
-  # prepare data of fixed effects and smooth terms for use in Stan
-  # handle smooth terms here as they also affect the FE design matrix 
-  # Args: see data_predictor
   out <- list()
   p <- usc(combine_prefix(bterms))
-  is_ordinal <- is_ordinal(bterms$family)
-  is_bsts <- is.cor_bsts(bterms$autocor)
   # the intercept is removed inside the Stan code for ordinal models
-  cols2remove <- if (is_ordinal && not4stan || is_bsts) "(Intercept)"
+  cols2remove <- if (is_ordinal(bterms)) "(Intercept)"
   X <- get_model_matrix(rhs(bterms$fe), data, cols2remove = cols2remove)
   avoid_dpars(colnames(X), bterms = bterms)
   out[[paste0("K", p)]] <- ncol(X)
@@ -113,8 +107,8 @@ data_fe <- function(bterms, data, not4stan = FALSE) {
   out
 }
 
+# data preparation for splines
 data_sm <- function(bterms, data, knots = NULL, smooths = NULL) {
-  # data preparation for splines
   out <- list()
   smterms <- all_terms(bterms[["sm"]])
   if (!length(smterms)) {
@@ -127,7 +121,8 @@ data_sm <- function(bterms, data, knots = NULL, smooths = NULL) {
     for (i in seq_along(smterms)) {
       smooths[[i]] <- smoothCon(
         eval2(smterms[i]), data = data, 
-        knots = knots, absorb.cons = TRUE
+        knots = knots, absorb.cons = TRUE,
+        diagonal.penalty = TRUE
       )
     }
   }
@@ -169,9 +164,8 @@ data_sm <- function(bterms, data, knots = NULL, smooths = NULL) {
   out
 }
 
+# prepare data for group-level effects for use in Stan
 data_re <- function(bterms, data, ranef) {
-  # prepare data for group-level effects for use in Stan
-  # Args: see data_predictor
   out <- list()
   px <- check_prefix(bterms)
   take <- find_rows(ranef, ls = px) & !find_rows(ranef, type = "sp")
@@ -231,10 +225,8 @@ data_re <- function(bterms, data, ranef) {
   out
 }
 
+# compute data for each group-level-ID per univariate model
 data_gr_local <- function(bterms, data, ranef) {
-  # compute data for each group-level-ID per univariate model
-  # Args:
-  #   ranef: data.frame returned by tidy_ranef
   stopifnot(is.brmsterms(bterms))
   out <- list()
   ranef <- subset2(ranef, resp = bterms$resp)
@@ -292,11 +284,8 @@ data_gr_local <- function(bterms, data, ranef) {
   out
 }
 
+# prepare global data for each group-level-ID
 data_gr_global <- function(ranef, cov_ranef = NULL) {
-  # prepare global data for each group-level-ID
-  # Args:
-  #   ranef: data.frame returned by tidy_ranef
-  #   cov_ranef: name list of user-defined covariance matrices
   out <- list()
   for (id in unique(ranef$id)) {
     id_ranef <- subset2(ranef, id = id)
@@ -342,9 +331,8 @@ data_gr_global <- function(ranef, cov_ranef = NULL) {
   out
 }
 
+# prepare data for special effects for use in Stan
 data_sp <- function(bterms, data, prior = brmsprior(), Jmo = NULL) {
-  # prepare data for special effects for use in Stan
-  # Args: see data_predictor
   out <- list()
   spef <- tidy_spef(bterms, data)
   if (!nrow(spef)) return(out)
@@ -363,10 +351,9 @@ data_sp <- function(bterms, data, prior = brmsprior(), Jmo = NULL) {
   if (any(lengths(spef$Imo) > 0)) {
     # prepare data specific to monotonic effects
     out[[paste0("Imo", p)]] <- max(unlist(spef$Imo))
-    Xmo_fun <- function(x) as.array(attr(eval2(x, data), "var"))
-    Xmo <- lapply(unlist(spef$calls_mo), Xmo_fun)
+    Xmo <- lapply(unlist(spef$calls_mo), get_mo_values, data = data)
     Xmo_names <- paste0("Xmo", p, "_", seq_along(Xmo))
-    out <- c(out, setNames(Xmo, Xmo_names))
+    c(out) <- setNames(Xmo, Xmo_names)
     compute_Jmo <- is.null(Jmo)
     if (!length(Jmo)) {
       Jmo <- as.array(ulapply(Xmo, max))
@@ -394,9 +381,8 @@ data_sp <- function(bterms, data, prior = brmsprior(), Jmo = NULL) {
   out
 }
 
+# prepare data for category specific effects
 data_cs <- function(bterms, data) {
-  # prepare data for category specific effects
-  # Args: see data_predictor
   out <- list()
   if (length(all_terms(bterms[["cs"]]))) {
     p <- usc(combine_prefix(bterms))
@@ -408,8 +394,8 @@ data_cs <- function(bterms, data) {
   out
 }
 
+# prepare global data for noise free variables
 data_Xme <- function(meef, data) {
-  # prepare global data for noise free variables
   stopifnot(is.meef_frame(meef))
   out <- list()
   groups <- unique(meef$grname)
@@ -421,7 +407,7 @@ data_Xme <- function(meef, data) {
     out[[paste0("NCme_", i)]] <- Mme * (Mme - 1) / 2
     if (nzchar(g)) {
       levels <- get_levels(meef)[[g]]
-      gr <- attributes(eval2(meef$term[K[1]], data))[["gr"]]
+      gr <- get_me_group(meef$term[K[1]], data)
       Jme <- match(gr, levels)
       if (anyNA(Jme)) {
         # occurs for new levels only
@@ -436,9 +422,8 @@ data_Xme <- function(meef, data) {
       out[[paste0("Jme_", i)]] <- Jme
     }
     for (k in K) {
-      att <- attributes(eval2(meef$term[k], data))
-      Xn <- as.array(att$var)
-      noise <- as.array(att$sdx)
+      Xn <- get_me_values(meef$term[k], data)
+      noise <- get_me_noise(meef$term[k], data)
       if (nzchar(g)) {
         for (l in ilevels) {
           # validate values of the same level
@@ -464,11 +449,10 @@ data_Xme <- function(meef, data) {
   out
 }
 
+# prepare data for Gaussian process terms
+# @param raw store certain intermediate data for further processing?
+# @param ... passed to '.data_gp'
 data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
-  # prepare data for Gaussian process terms
-  # Args: see data_predictor
-  #   raw: store certain intermediate data for further processing?
-  #   ...: passed to '.data_gp'
   out <- list()
   raw <- as_one_logical(raw)
   px <- check_prefix(bterms)
@@ -551,16 +535,15 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
   out
 }
 
+# helper function to preparae GP related data
+# @inheritParams data_gp
+# @param Xgp matrix of covariate values
+# @param k, gr, c see 'tidy_gpef'
+# @param sfx suffix to put at the end of data names
+# @param Cgp optional vector of values belonging to
+#   a certain contrast of a factor 'by' variable
 .data_gp <- function(Xgp, k, gr, sfx, Cgp = NULL, c = NULL, 
                      raw = FALSE, gps = NULL) {
-  # helper function to preparae GP related data
-  # Args:
-  #   Xgp: matrix of covariate values
-  #   k, gr, c: see tidy_gpef
-  #   sfx: suffix to put at the end of data names
-  #   Cgp: optional vector of values belonging to
-  #     a certain contrast of a factor 'by' variable
-  #   raw, gps: see 'data_gp'
   out <- list()
   if (!is.null(Cgp)) {
     Cgp <- unname(Cgp)
@@ -616,9 +599,8 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
   out
 }
 
+# prepare data of offsets for use in Stan
 data_offset <- function(bterms, data) {
-  # prepare data of offsets for use in Stan
-  # Args: see data_predictor
   out <- list()
   px <- check_prefix(bterms)
   if (is.formula(bterms$offset)) {
@@ -634,57 +616,43 @@ data_offset <- function(bterms, data) {
   out
 }
 
+# data for autocorrelation variables
+# @param Y vector of response values; only required in cor_arr
+# @param new: does 'data' contain new data?
+# @param old_locations: optional old locations for CAR models
 data_autocor <- function(bterms, data, Y = NULL, new = FALSE,
                          old_locations = NULL) {
-  # data for autocorrelation variables
-  # Args:
-  #   Y: vector of response values; only required in cor_arr
-  #   new: does 'data' contain new data?
-  #   old_locations: optional locations for CAR models 
-  #     used when fitting the model
   stopifnot(is.brmsterms(bterms))
   autocor <- bterms$autocor
   N <- nrow(data)
   out <- list()
-  if (is.cor_arma(autocor) || is.cor_bsts(autocor)) {
+  if (is.cor_arma(autocor)) {
+    # ARMA correlations
     if (length(bterms$time$group)) {
       tgroup <- as.numeric(factor(data[[bterms$time$group]]))
     } else {
       tgroup <- rep(1, N) 
     }
-  }
-  if (has_arma(autocor)) {
-    Kar <- get_ar(autocor)
-    Kma <- get_ma(autocor)
-    Karr <- get_arr(autocor)
-    if (Kar || Kma) {
-      # ARMA correlations (of residuals)
-      out$Kar <- Kar
-      out$Kma <- Kma
-      if (use_cov(autocor)) {
-        # data for the 'covariance' version of ARMA 
-        out$N_tg <- length(unique(tgroup))
-        out$begin_tg <- as.array(ulapply(unique(tgroup), match, tgroup))
-        out$nobs_tg <- as.array(with(out, 
-          c(if (N_tg > 1L) begin_tg[2:N_tg], N + 1) - begin_tg
-        ))
-        out$end_tg <- with(out, begin_tg + nobs_tg - 1)
-      } else {
-        # data for the 'predictor' version of ARMA
-        max_lag <- max(Kar, Kma)
-        out$J_lag <- as.array(rep(0, N))
-        for (n in seq_len(N)[-N]) {
-          ind <- n:max(1, n + 1 - max_lag)
-          # indexes errors to be used in the n+1th prediction
-          out$J_lag[n] <- sum(tgroup[ind] %in% tgroup[n + 1])
-        }
+    out$Kar <- get_ar(autocor)
+    out$Kma <- get_ma(autocor)
+    if (use_cov(autocor)) {
+      # data for the 'covariance' version of ARMA 
+      out$N_tg <- length(unique(tgroup))
+      out$begin_tg <- as.array(ulapply(unique(tgroup), match, tgroup))
+      out$nobs_tg <- as.array(with(out, 
+        c(if (N_tg > 1L) begin_tg[2:N_tg], N + 1) - begin_tg
+      ))
+      out$end_tg <- with(out, begin_tg + nobs_tg - 1)
+    } else {
+      # data for the 'predictor' version of ARMA
+      max_lag <- max(out$Kar, out$Kma)
+      out$J_lag <- as.array(rep(0, N))
+      for (n in seq_len(N)[-N]) {
+        ind <- n:max(1, n + 1 - max_lag)
+        # indexes errors to be used in the n+1th prediction
+        out$J_lag[n] <- sum(tgroup[ind] %in% tgroup[n + 1])
       }
     }
-    if (Karr) {
-      # ARR effects (autoregressive effects of the response)
-      out$Yarr <- arr_design_matrix(Y, Karr, tgroup)
-      out$Karr <- Karr
-    }
   } else if (is.cor_sar(autocor)) {
     if (!identical(dim(autocor$W), rep(N, 2))) {
       stop2("Dimensions of 'W' must be equal to the number of observations.")
@@ -752,8 +720,6 @@ data_autocor <- function(bterms, data, Y = NULL, new = FALSE,
       eigenW <- eigen(eigenW, TRUE, only.values = TRUE)$values
       c(out) <- nlist(Nneigh, eigenW)
     }
-  } else if (is.cor_bsts(autocor)) {
-    out$tg <- as.array(tgroup)
   } else if (is.cor_fixed(autocor)) {
     V <- autocor$V
     rmd_rows <- attr(data, "na.action")
@@ -865,19 +831,21 @@ data_response.brmsterms <- function(x, data, check_response = TRUE,
     ybounds <- family_info(x$family, "ybounds")
     closed <- family_info(x$family, "closed")
     if (is.finite(ybounds[1])) {
-      if (closed[1] && min(out$Y) < ybounds[1]) {
+      y_min <- min(out$Y, na.rm = TRUE)
+      if (closed[1] && y_min < ybounds[1]) {
         stop2("Family '", family4error, "' requires response greater ",
               "than or equal to ", ybounds[1], ".")
-      } else if (!closed[1] && min(out$Y) <= ybounds[1]) {
+      } else if (!closed[1] && y_min <= ybounds[1]) {
         stop2("Family '", family4error, "' requires response greater ",
               "than ", round(ybounds[1], 2), ".")
       }
     }
     if (is.finite(ybounds[2])) {
-      if (closed[2] && max(out$Y) > ybounds[2]) {
+      y_max <- max(out$Y, na.rm = TRUE)
+      if (closed[2] && y_max > ybounds[2]) {
         stop2("Family '", family4error, "' requires response smaller ",
               "than or equal to ", ybounds[2], ".")
-      } else if (!closed[2] && max(out$Y) >= ybounds[2]) {
+      } else if (!closed[2] && y_max >= ybounds[2]) {
         stop2("Family '", family4error, "' requires response smaller ",
               "than ", round(ybounds[2], 2), ".")
       }
@@ -1015,11 +983,14 @@ data_response.brmsterms <- function(x, data, check_response = TRUE,
       out$Jme <- as.array(setdiff(seq_along(out$Y), which_mi))
       out$Nme <- length(out$Jme)
       out$noise <- as.array(sdy)
+      if (!not4stan) {
+        out$noise[which_mi] <- Inf
+      }
     }
     if (!not4stan) {
       # Stan does not allow NAs in data
       # use Inf to that min(Y) is not affected
-      out$Y[which_mi] <- out$noise[which_mi] <- Inf
+      out$Y[which_mi] <- Inf
     }
   } 
   resp <- usc(combine_prefix(x))
@@ -1032,8 +1003,8 @@ data_response.brmsterms <- function(x, data, check_response = TRUE,
   out
 }
 
+# data specific for mixture models
 data_mixture <- function(bterms, prior = brmsprior()) {
-  # data specific for mixture models
   stopifnot(is.brmsterms(bterms))
   out <- list()
   if (is.mixfamily(bterms$family)) {
@@ -1060,8 +1031,8 @@ data_mixture <- function(bterms, prior = brmsprior()) {
   out
 }
 
+# data for special priors such as horseshoe and lasso
 data_prior <- function(bterms, data, prior) {
-  # data for special priors such as horseshoe and lasso
   out <- list()
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
