@@ -96,15 +96,14 @@ resp_se <- function(x, sigma = FALSE) {
   structure(x, sigma = sigma)  
 }
 
+# only evaluate the sigma argument of 'resp_se'
 resp_se_no_data <- function(x, sigma = FALSE) {
-  # only evaluate the sigma argument
   resp_se(1, sigma = sigma)
 }
 
 #' @rdname addition-terms
 #' @export
 resp_weights <- function(x, scale = FALSE) {
-  # weights to be applied on any model
   if (!is.numeric(x)) {
     stop2("Weights must be numeric.")
   }
@@ -121,7 +120,6 @@ resp_weights <- function(x, scale = FALSE) {
 #' @rdname addition-terms
 #' @export
 resp_trials <- function(x) {
-  # trials for binomial models
   if (!is.numeric(x)) {
     stop2("Number of trials must be numeric.")
   }
@@ -134,7 +132,6 @@ resp_trials <- function(x) {
 #' @rdname addition-terms
 #' @export
 resp_cat <- function(x) {
-  # number of categories for ordinal models
   x <- as_one_numeric(x)
   if (!is_wholenumber(x) || x < 1) {
     stop2("Number of categories must be a positive integer.")
@@ -145,7 +142,6 @@ resp_cat <- function(x) {
 #' @rdname addition-terms
 #' @export
 resp_dec <- function(x) {
-  # decisions for the wiener diffusion model
   if (is.character(x) || is.factor(x)) {
     if (!all(unique(x) %in% c("lower", "upper"))) {
       stop2("Decisions should be 'lower' or 'upper' ", 
@@ -161,7 +157,6 @@ resp_dec <- function(x) {
 #' @rdname addition-terms
 #' @export
 resp_cens <- function(x, y2 = NULL) {
-  # indicator for censoring
   if (is.factor(x)) {
     x <- as.character(x)
   }
@@ -268,12 +263,47 @@ t2 <- function(...) {
   mgcv::t2(...)
 }
 
-#' Predictors with Measurement Error in \pkg{brms} Models
+#' Category Specific Predictors in \pkg{brms} Models
+#' 
+#' @aliases cse
+#' 
+#' @param expr Expression containing predictors,
+#'  for which category specific effects should be estimated. 
+#'  For evaluation, \R formula syntax is applied.
+#'  
+#' @details For detailed documentation see \code{help(brmsformula)}
+#'   as well as \code{vignette("brms_overview")}.
 #' 
-#' Specify predictors with measurement error.
 #' This function is almost solely useful when
 #' called in formulas passed to the \pkg{brms} package.
 #' 
+#' @seealso \code{\link{brmsformula}}
+#'   
+#' @examples   
+#' \dontrun{
+#' fit <- brm(rating ~ period + carry + cs(treat), 
+#'            data = inhaler, family = sratio("cloglog"), 
+#'            prior = set_prior("normal(0,5)"), chains = 2)
+#' summary(fit)
+#' plot(fit, ask = FALSE)
+#' } 
+#'  
+#' @export
+cs <- function(expr) {
+  deparse_no_string(substitute(expr))
+}
+
+# alias of function 'cs' used in the JSS paper of brms
+#' @export
+cse <- function(expr) {
+  deparse_no_string(substitute(expr))
+}
+
+#' Predictors with Measurement Error in \pkg{brms} Models
+#' 
+#' Specify predictors with measurement error. The function does not evaluate its
+#' arguments -- it exists purely to help set up a model.
+#' 
 #' @param x The variable measured with error.
 #' @param sdx Known measurement error of \code{x}
 #'   treated as standard deviation.
@@ -312,49 +342,32 @@ t2 <- function(...) {
 #' } 
 #' 
 #' @export
-me <- function(x, sdx = NULL, gr = NULL) {
-  xname <- deparse(substitute(x))
-  x <- as.vector(x)
-  sdx <- as.vector(sdx)
-  if (length(sdx) == 0L) {
-    stop2("Argument 'sdx' is missing in function 'me'.")
-  } else if (length(sdx) == 1L) {
-    sdx <- rep(sdx, length(x))
-  }
-  if (!is.numeric(x)) {
-    stop2("Noisy variables should be numeric.")
-  }
-  if (!is.numeric(sdx)) {
-    stop2("Measurement error should be numeric.")
-  }
-  if (isTRUE(any(sdx <= 0))) {
-    stop2("Measurement error should be positive.")
-  }
-  grname <- substitute(gr)
-  if (!is.null(grname)) {
-    grname <- deparse_combine(grname, max_char = NULL)
-    stopif_illegal_group(grname)
-    gr <- as.vector(gr)
+me <- function(x, sdx, gr = NULL) {
+  # use 'term' for consistency with other special terms
+  term <- deparse(substitute(x))
+  sdx <- deparse(substitute(sdx))
+  gr <- substitute(gr)
+  if (!is.null(gr)) {
+    gr <- deparse_combine(gr)
+    stopif_illegal_group(gr)
   } else {
-    grname <- ""
-    gr <- character(0)
+    gr <- ""
   }
-  out <- rep(1, length(x))
-  structure(out, 
-    var = x, sdx = sdx, gr = gr, 
-    xname = xname, grname = grname
-  )
+  label <- deparse(match.call())
+  out <- nlist(term, sdx, gr, label)
+  class(out) <- c("me_term", "sp_term")
+  out
 }
 
 #' Predictors with Missing Values in \pkg{brms} Models
 #' 
+#' Specify predictor term with missing values in \pkg{brms}. The function does
+#' not evaluate its arguments -- it exists purely to help set up a model.
+#' 
 #' @param x The variable containing missings.
 #' 
 #' @details For detailed documentation see \code{help(brmsformula)}. 
 #' 
-#' This function is almost solely useful when
-#' called in formulas passed to the \pkg{brms} package.
-#' 
 #' @seealso \code{\link{brmsformula}}
 #'   
 #' @examples 
@@ -370,70 +383,35 @@ me <- function(x, sdx = NULL, gr = NULL) {
 #' 
 #' @export
 mi <- function(x) {
-  xname <- substitute(x)
-  vars <- all.vars(xname)
-  xname <- deparse(xname)
-  if (!is_equal(xname, vars)) {
-    stop2("Function 'mi' can only handle single untransformed variables.")
+  # use 'term' for consistency with other special terms
+  term <- substitute(x)
+  vars <- all.vars(term)
+  term <- deparse(term)
+  if (!is_equal(term, vars)) {
+    stop2("'mi' only accepts single untransformed variables.")
   }
-  x <- as.vector(x)
-  if (!is.numeric(x)) {
-    stop2("Noisy variables should be numeric.")
-  }
-  out <- rep(1, length(x))
-  structure(out, var = x, xname = xname)
-}
-
-#' Category Specific Predictors in \pkg{brms} Models
-#' 
-#' @aliases cse
-#' 
-#' @param expr Expression containing predictors,
-#'  for which category specific effects should be estimated. 
-#'  For evaluation, \R formula syntax is applied.
-#'  
-#' @details For detailed documentation see \code{help(brmsformula)}
-#'   as well as \code{vignette("brms_overview")}.
-#' 
-#' This function is almost solely useful when
-#' called in formulas passed to the \pkg{brms} package.
-#' 
-#' @seealso \code{\link{brmsformula}}
-#'   
-#' @examples   
-#' \dontrun{
-#' fit <- brm(rating ~ period + carry + cs(treat), 
-#'            data = inhaler, family = sratio("cloglog"), 
-#'            prior = set_prior("normal(0,5)"), chains = 2)
-#' summary(fit)
-#' plot(fit, ask = FALSE)
-#' } 
-#'  
-#' @export
-cs <- function(expr) {
-  deparse_no_string(substitute(expr))
-}
-
-#' @export
-cse <- function(expr) {
-  # alias of function 'cs'
-  deparse_no_string(substitute(expr))
+  label <- deparse(match.call())
+  out <- nlist(term, label)
+  class(out) <- c("mi_term", "sp_term")
+  out
 }
 
 #' Monotonic Predictors in \pkg{brms} Models
 #' 
-#' @aliases mono monotonic
+#' Specify a monotonic predictor term in \pkg{brms}. The function does not
+#' evaluate its arguments -- it exists purely to help set up a model.
 #' 
-#' @param x An integer variable or an ordered factor
-#'   to be modeled as monotonic.
+#' @param x An integer variable or an ordered factor to be modeled as monotonic.
 #'  
 #' @details For detailed documentation see \code{help(brmsformula)}
 #'   as well as \code{vignette("brms_monotonic")}.
 #' 
-#' This function is almost solely useful when
-#' called in formulas passed to the \pkg{brms} package.
-#' 
 #' @seealso \code{\link{brmsformula}}
+#' 
+#' @references 
+#' Bürkner P. C. & Charpentier, E. (in review). Monotonic Effects: A Principled 
+#' Approach for Including Ordinal Predictors in Regression Models. PsyArXiv 
+#' preprint.
 #'   
 #' @examples   
 #' \dontrun{
@@ -464,39 +442,12 @@ cse <- function(expr) {
 #'  
 #' @export
 mo <- function(x) {
-  x_name <- attr(x, "x_name")
-  if (is.null(x_name)) {
-    x_name <- deparse_combine(substitute(x))  
-  }
-  if (is.ordered(x)) {
-    # counting starts at zero
-    x <- as.numeric(x) - 1 
-  } else if (all(is_wholenumber(x))) {
-    min_value <- attr(x, "min")
-    if (is.null(min_value)) {
-      min_value <- min(x)
-    }
-    x <- x - min_value
-  } else {
-    stop2(
-      "Monotonic predictors must be integers or ordered ", 
-      "factors. Error occured for variable '", x_name, "'."
-    )
-  }
-  out <- rep(1, length(x))
-  structure(out, var = x) 
-}
-
-#' @export
-mono <- function(x) {
-  attr(x, "x_name") <- deparse_combine(substitute(x)) 
-  mo(x)
-}
-
-#' @export
-monotonic <- function(x) {
-  attr(x, "x_name") <- deparse_combine(substitute(x))
-  mo(x)
+  # use 'term' for consistency with other special terms
+  term <- deparse(substitute(x))
+  label <- deparse(match.call())
+  out <- nlist(term, label)
+  class(out) <- c("mo_term", "sp_term")
+  out
 }
 
 #' Set up Gaussian process terms in \pkg{brms}
@@ -557,7 +508,7 @@ monotonic <- function(x) {
 #'  Below, we describe the currently supported covariance kernels:
 #'  \itemize{
 #'    \item{"exp_quad": }{The exponentiated-quadratic kernel is defined as
-#'    \eqn{k(x_i, x_j) = sdgp^2 exp(- || x_i - x_j || / (2 lscale^2))},
+#'    \eqn{k(x_i, x_j) = sdgp^2 exp(- || x_i - x_j ||^2 / (2 lscale^2))},
 #'    where \eqn{|| . ||} is the Euclidean norm, \eqn{sdgp} is a 
 #'    standard deviation parameter, and \eqn{lscale} is characteristic 
 #'    length-scale parameter. The latter practically measures how close two 
@@ -817,16 +768,16 @@ mmc <- function(...) {
   out
 }
 
+# return the right-hand side of a formula
 rhs <- function(x) {
-  # return the righthand side of a formula
   attri <- attributes(x)
   x <- as.formula(x)
   x <- if (length(x) == 3) x[-2] else x
   do_call(structure, c(list(x), attri))
 }
 
+# return the left-hand side of a formula
 lhs <- function(x) {
-  # return the lefthand side of a formula
   x <- as.formula(x)
   if (length(x) == 3L) update(x, . ~ 1) else NULL
 }
@@ -837,13 +788,10 @@ eval_rhs <- function(formula, data = NULL) {
   eval(rhs(formula)[[2]], data, environment(formula))
 }
 
+# convert a string to a formula
+# @param x vector of strings to be converted
+# @param ... passed to formula()
 str2formula <- function(x, ..., collapse = "+") {
-  # converts a string to a formula
-  # Args:
-  #   x: vector of strings to be converted
-  #   ...: passed to formula(.)
-  # Returns:
-  #   a formula
   has_chars <- nzchar(x)
   if (length(x) && any(has_chars)) {
     out <- paste(x[has_chars], collapse = collapse) 
@@ -855,21 +803,15 @@ str2formula <- function(x, ..., collapse = "+") {
   out
 }
 
+# convert a formula to a character string
+# @param formula a model formula
+# @param rm a vector of to elements indicating how many characters 
+#   should be removed at the beginning and end of the string respectively
+# @param space how should whitespaces be treated?
 formula2str <- function(formula, rm = c(0, 0), space = c("rm", "trim")) {
-  # converts a formula to a string
-  # Args:
-  #   formula: a model formula
-  #   rm: a vector of to elements indicating how many characters 
-  #       should be removed at the beginning
-  #       and end of the string respectively
-  #   space: how should whitespaces be treated?
-  # Returns:
-  #   a string
+  formula <- as.formula(formula)
   space <- match.arg(space)
-  if (!is.formula(formula)) {
-    formula <- as.formula(formula)
-  }
-  if (is.na(rm[2])) rm[2] <- 0
+  if (anyNA(rm[2])) rm[2] <- 0
   x <- Reduce(paste, deparse(formula))
   x <- gsub("[\t\r\n]+", "", x, perl = TRUE)
   if (space == "trim") {
@@ -884,8 +826,8 @@ is.formula <- function(x) {
   inherits(x, "formula")
 }
 
+# expand the '.' variable in formula using stats::terms
 expand_dot_formula <- function(formula, data = NULL) {
-  # expand the '.' variable in formula using stats::terms
   if (isTRUE("." %in% all.vars(formula))) {
     att <- attributes(formula)
     try_terms <- try(
@@ -899,23 +841,3 @@ expand_dot_formula <- function(formula, data = NULL) {
   }
   formula
 }
-
-extract_cat_names <- function(x, data) {
-  # extract names of response categories
-  stopifnot(is.brmsformula(x) || is.brmsterms(x))
-  respform <- formula2str(lhs(x$formula))
-  respform <- formula(gsub("\\|+[^~]*~", "~", respform))
-  mr <- model.response(model.frame(respform, data))
-  if (is_ordinal(x) && is.numeric(mr)) {
-    out <- as.character(seq_len(max(mr)))
-  } else if (has_multicol(x)) {
-    mr <- as.matrix(mr)
-    out <- as.character(colnames(mr))
-    if (!length(out)) {
-      out <- as.character(seq_cols(mr))
-    }
-  } else {
-    out <- levels(factor(mr))
-  }
-  out
-}
