@@ -1,5 +1,5 @@
+# evaluate log_lik for jointly for all observations 
 log_lik_internal <- function(draws, ...) {
-  # for use in non-pointwise evaluation only
   UseMethod("log_lik_internal")
 }
 
@@ -32,16 +32,16 @@ log_lik_internal.brmsdraws <- function(draws, ...) {
     draws$dpars[[dp]] <- get_dpar(draws, dpar = dp)
   }
   N <- choose_N(draws)
-  out <- do_call(cbind, lapply(seq_len(N), log_lik_fun, draws = draws))
+  out <- cblapply(seq_len(N), log_lik_fun, draws = draws)
   colnames(out) <- NULL
   old_order <- draws$old_order
   sort <- isTRUE(ncol(out) != length(old_order))
   reorder_obs(out, old_order, sort = sort)
 }
 
+# evaluate log_lik in a pointwise manner
+# cannot be an S3 method since 'i' must be the first argument
 log_lik_pointwise <- function(data_i, draws, ...) {
-  # for use in pointwise evaluation only
-  # cannot be made an S3 methods since i must be the first argument
   i <- data_i$i
   if (is.mvbrmsdraws(draws) && !length(draws$mvpars$rescor)) {
     out <- lapply(draws$resps, log_lik_pointwise, i = i)
@@ -55,14 +55,12 @@ log_lik_pointwise <- function(data_i, draws, ...) {
 }
 
 # All log_lik_<family> functions have the same arguments structure
-# Args:
-#  i: the column of draws to use i.e. the ith obervation 
-#     in the initial data.frame 
-#  draws: A named list returned by extract_draws containing 
-#         all required data and samples
-#  data: ignored; included for compatibility with loo::loo.function      
-# Returns:
-#   A vector of length draws$nsamples containing the pointwise 
+# @param i the column of draws to use the the ith obervation 
+#   in the initial data.frame 
+# @param draws A named list returned by extract_draws containing 
+#   all required data and samples
+# @param data (ignored) included for compatibility with loo::loo.function      
+# @return a vector of length draws$nsamples containing the pointwise 
 #   log-likelihood fo the ith observation 
 log_lik_gaussian <- function(i, draws, data = data.frame()) {
   args <- list(
@@ -70,11 +68,11 @@ log_lik_gaussian <- function(i, draws, data = data.frame()) {
     sd = get_dpar(draws, "sigma", i = i)
   )
   # log_lik_censor computes the conventional log_lik in case of no censoring 
-  out <- log_lik_censor(dist = "norm", args = args, i = i, data = draws$data)
+  out <- log_lik_censor(dist = "norm", args = args, i = i, draws = draws)
   out <- log_lik_truncate(
-    out, cdf = pnorm, args = args, i = i, data = draws$data
+    out, cdf = pnorm, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_student <- function(i, draws, data = data.frame()) {
@@ -84,31 +82,31 @@ log_lik_student <- function(i, draws, data = data.frame()) {
     sigma = get_dpar(draws, "sigma", i = i)
   )
   out <- log_lik_censor(
-    dist = "student_t", args = args, i = i, data = draws$data
+    dist = "student_t", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pstudent_t, args = args, i = i, data = draws$data
+    out, cdf = pstudent_t, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_lognormal <- function(i, draws, data = data.frame()) {
   sigma <- get_dpar(draws, "sigma", i = i)
   args <- list(meanlog = get_dpar(draws, "mu", i), sdlog = sigma)
-  out <- log_lik_censor(dist = "lnorm", args = args, i = i, data = draws$data)
+  out <- log_lik_censor(dist = "lnorm", args = args, i = i, draws = draws)
   out <- log_lik_truncate(
-    out, cdf = plnorm, args = args, i = i, data = draws$data
+    out, cdf = plnorm, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_shifted_lognormal <- function(i, draws, data = data.frame()) {
   sigma <- get_dpar(draws, "sigma", i = i)
   ndt <- get_dpar(draws, "ndt", i = i)
   args <- list(meanlog = get_dpar(draws, "mu", i), sdlog = sigma, shift = ndt)
-  out <- log_lik_censor("shifted_lnorm", args, i = i, data = draws$data)
-  out <- log_lik_truncate(out, pshifted_lnorm, args, i = i, data = draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("shifted_lnorm", args, i = i, draws = draws)
+  out <- log_lik_truncate(out, pshifted_lnorm, args, i = i, draws = draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_skew_normal <- function(i, draws, data = data.frame()) {
@@ -117,12 +115,12 @@ log_lik_skew_normal <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   args <- nlist(mu, sigma, alpha)
   out <- log_lik_censor(
-    dist = "skew_normal", args = args, i = i, data = draws$data
+    dist = "skew_normal", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pskew_normal, args = args, i = i, data = draws$data
+    out, cdf = pskew_normal, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_gaussian_mv <- function(i, draws, data = data.frame()) {
@@ -135,7 +133,7 @@ log_lik_gaussian_mv <- function(i, draws, data = data.frame()) {
     )
   }
   out <- sapply(1:draws$nsamples, dmn)
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_student_mv <- function(i, draws, data = data.frame()) {
@@ -149,11 +147,11 @@ log_lik_student_mv <- function(i, draws, data = data.frame()) {
     )
   }
   out <- sapply(1:draws$nsamples, dmst)
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_gaussian_cov <- function(i, draws, data = data.frame()) {
-  obs <- with(draws$ac, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
+  obs <- with(draws$ac, begin_tg[i]:end_tg[i])
   Y <- as.numeric(draws$data$Y[obs])
   mu <- as.matrix(get_dpar(draws, "mu", i = obs))
   Sigma <- get_cov_matrix_arma(draws, obs)
@@ -166,8 +164,7 @@ log_lik_gaussian_cov <- function(i, draws, data = data.frame()) {
     ll <- dnorm(Y, yloo, sdloo, log = TRUE)
     return(as.numeric(ll))
   }
-  out <- lapply(seq_len(draws$nsamples), .log_lik)
-  do_call(rbind, out)
+  rblapply(seq_len(draws$nsamples), .log_lik)
 }
 
 log_lik_student_cov <- function(i, draws, data = data.frame()) {
@@ -175,12 +172,12 @@ log_lik_student_cov <- function(i, draws, data = data.frame()) {
 }
 
 log_lik_gaussian_lagsar <- function(i, draws, data = data.frame()) {
-  # see http://mc-stan.org/loo/articles/loo2-non-factorizable.html
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
   Y <- as.numeric(draws$data$Y)
   I <- diag(draws$nobs)
   stopifnot(i == 1)
+  # see http://mc-stan.org/loo/articles/loo2-non-factorizable.html
   .log_lik <- function(s) {
     IB <- I - with(draws$ac, lagsar[s, ] * W)
     Cinv <- t(IB) %*% IB / sigma[s]^2
@@ -191,8 +188,7 @@ log_lik_gaussian_lagsar <- function(i, draws, data = data.frame()) {
     ll <- dnorm(Y, yloo, sdloo, log = TRUE)
     return(as.numeric(ll))
   }
-  out <- lapply(seq_len(draws$nsamples), .log_lik)
-  do_call(rbind, out)
+  rblapply(seq_len(draws$nsamples), .log_lik)
 }
 
 log_lik_student_lagsar <- function(i, draws, data = data.frame()) {
@@ -215,8 +211,7 @@ log_lik_gaussian_errorsar <- function(i, draws, data = data.frame()) {
     ll <- dnorm(Y, yloo, sdloo, log = TRUE)
     return(as.numeric(ll))
   }
-  out <- lapply(seq_len(draws$nsamples), .log_lik)
-  do_call(rbind, out)
+  rblapply(seq_len(draws$nsamples), .log_lik)
 }
 
 log_lik_student_errorsar <- function(i, draws, data = data.frame()) {
@@ -236,8 +231,7 @@ log_lik_gaussian_fixed <- function(i, draws, data = data.frame()) {
     ll <- dnorm(Y, yloo, sdloo, log = TRUE)
     return(as.numeric(ll))
   }
-  out <- lapply(seq_len(draws$nsamples), .log_lik)
-  do_call(rbind, out)
+  rblapply(seq_len(draws$nsamples), .log_lik)
 }
 
 log_lik_student_fixed <- function(i, draws, data = data.frame()) {
@@ -248,55 +242,55 @@ log_lik_binomial <- function(i, draws, data = data.frame()) {
   trials <- draws$data$trials[i]
   args <- list(size = trials, prob = get_dpar(draws, "mu", i))
   out <- log_lik_censor(
-    dist = "binom", args = args, i = i, data = draws$data
+    dist = "binom", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pbinom, args = args, i = i, data = draws$data
+    out, cdf = pbinom, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }  
 
 log_lik_bernoulli <- function(i, draws, data = data.frame()) {
   args <- list(size = 1, prob = get_dpar(draws, "mu", i))
   out <- log_lik_censor(
-    dist = "binom", args = args, i = i, data = draws$data
+    dist = "binom", args = args, i = i, draws = draws
   )
   # no truncation allowed
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_poisson <- function(i, draws, data = data.frame()) {
   args <- list(lambda = get_dpar(draws, "mu", i))
   out <- log_lik_censor(
-    dist = "pois", args = args, i = i, data = draws$data
+    dist = "pois", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = ppois, args = args, i = i, data = draws$data
+    out, cdf = ppois, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_negbinomial <- function(i, draws, data = data.frame()) {
   shape <- get_dpar(draws, "shape", i = i)
   args <- list(mu = get_dpar(draws, "mu", i), size = shape)
   out <- log_lik_censor(
-    dist = "nbinom", args = args, i = i, data = draws$data
+    dist = "nbinom", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pnbinom, args = args, i = i, data = draws$data
+    out, cdf = pnbinom, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_geometric <- function(i, draws, data = data.frame()) {
   args <- list(mu = get_dpar(draws, "mu", i), size = 1)
   out <- log_lik_censor(
-    dist = "nbinom", args = args, i = i, data = draws$data
+    dist = "nbinom", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pnbinom, args = args, i = i, data = draws$data
+    out, cdf = pnbinom, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_discrete_weibull <- function(i, draws, data = data.frame()) {
@@ -305,12 +299,12 @@ log_lik_discrete_weibull <- function(i, draws, data = data.frame()) {
     shape = get_dpar(draws, "shape", i = i)
   )
   out <- log_lik_censor(
-    dist = "discrete_weibull", args = args, i = i, data = draws$data
+    dist = "discrete_weibull", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pdiscrete_weibull, args = args, i = i, data = draws$data
+    out, cdf = pdiscrete_weibull, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_com_poisson <- function(i, draws, data = data.frame()) {
@@ -320,27 +314,27 @@ log_lik_com_poisson <- function(i, draws, data = data.frame()) {
   )
   # no censoring or truncation allowed yet
   out <- do_call(dcom_poisson, c(draws$data$Y[i], args, log = TRUE))
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_exponential <- function(i, draws, data = data.frame()) {
   args <- list(rate = 1 / get_dpar(draws, "mu", i))
-  out <- log_lik_censor(dist = "exp", args = args, i = i, data = draws$data)
+  out <- log_lik_censor(dist = "exp", args = args, i = i, draws = draws)
   out <- log_lik_truncate(
-    out, cdf = pexp, args = args, i = i, data = draws$data
+    out, cdf = pexp, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_gamma <- function(i, draws, data = data.frame()) {
   shape <- get_dpar(draws, "shape", i = i)
   scale <- get_dpar(draws, "mu", i) / shape
   args <- nlist(shape, scale)
-  out <- log_lik_censor(dist = "gamma", args = args, i = i, data = draws$data)
+  out <- log_lik_censor(dist = "gamma", args = args, i = i, draws = draws)
   out <- log_lik_truncate(
-    out, cdf = pgamma, args = args, i = i, data = draws$data
+    out, cdf = pgamma, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_weibull <- function(i, draws, data = data.frame()) {
@@ -348,12 +342,12 @@ log_lik_weibull <- function(i, draws, data = data.frame()) {
   scale <- get_dpar(draws, "mu", i = i) / gamma(1 + 1 / shape)
   args <- list(shape = shape, scale = scale)
   out <- log_lik_censor(
-    dist = "weibull", args = args, i = i, data = draws$data
+    dist = "weibull", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pweibull, args = args, i = i, data = draws$data
+    out, cdf = pweibull, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_frechet <- function(i, draws, data = data.frame()) {
@@ -361,12 +355,12 @@ log_lik_frechet <- function(i, draws, data = data.frame()) {
   scale <- get_dpar(draws, "mu", i = i) / gamma(1 - 1 / nu)
   args <- list(scale = scale, shape = nu)
   out <- log_lik_censor(
-    dist = "frechet", args = args, i = i, data = draws$data
+    dist = "frechet", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pfrechet, args = args, i = i, data = draws$data
+    out, cdf = pfrechet, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_gen_extreme_value <- function(i, draws, data = data.frame()) {
@@ -375,20 +369,20 @@ log_lik_gen_extreme_value <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   args <- nlist(mu, sigma, xi)
   out <- log_lik_censor(dist = "gen_extreme_value", args = args, 
-                       i = i, data = draws$data)
+                       i = i, draws = draws)
   out <- log_lik_truncate(out, cdf = pgen_extreme_value, 
-                         args = args, i = i, data = draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+                         args = args, i = i, draws = draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_inverse.gaussian <- function(i, draws, data = data.frame()) {
   args <- list(mu = get_dpar(draws, "mu", i), 
                shape = get_dpar(draws, "shape", i = i))
   out <- log_lik_censor(dist = "inv_gaussian", args = args, 
-                       i = i, data = draws$data)
+                       i = i, draws = draws)
   out <- log_lik_truncate(out, cdf = pinv_gaussian, args = args,
-                         i = i, data = draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+                         i = i, draws = draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_exgaussian <- function(i, draws, data = data.frame()) {
@@ -396,10 +390,10 @@ log_lik_exgaussian <- function(i, draws, data = data.frame()) {
                sigma = get_dpar(draws, "sigma", i = i),
                beta = get_dpar(draws, "beta", i = i))
   out <- log_lik_censor(dist = "exgaussian", args = args, 
-                       i = i, data = draws$data)
+                       i = i, draws = draws)
   out <- log_lik_truncate(out, cdf = pexgaussian, args = args,
-                         i = i, data = draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+                         i = i, draws = draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_wiener <- function(i, draws, data = data.frame()) {
@@ -410,19 +404,19 @@ log_lik_wiener <- function(i, draws, data = data.frame()) {
     beta = get_dpar(draws, "bias", i = i),
     resp = draws$data[["dec"]][i]
   )
-  out <- do_call("dwiener", c(draws$data$Y[i], args, log = TRUE))
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- do_call(dwiener, c(draws$data$Y[i], args, log = TRUE))
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_beta <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   phi <- get_dpar(draws, "phi", i)
   args <- list(shape1 = mu * phi, shape2 = (1 - mu) * phi)
-  out <- log_lik_censor(dist = "beta", args = args, i = i, data = draws$data)
+  out <- log_lik_censor(dist = "beta", args = args, i = i, draws = draws)
   out <- log_lik_truncate(
-    out, cdf = pbeta, args = args, i = i, data = draws$data
+    out, cdf = pbeta, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_von_mises <- function(i, draws, data = data.frame()) {
@@ -431,12 +425,12 @@ log_lik_von_mises <- function(i, draws, data = data.frame()) {
     kappa = get_dpar(draws, "kappa", i = i)
   )
   out <- log_lik_censor(
-    dist = "von_mises", args = args, i = i, data = draws$data
+    dist = "von_mises", args = args, i = i, draws = draws
   )
   out <- log_lik_truncate(
-    out, cdf = pvon_mises, args = args, i = i, data = draws$data
+    out, cdf = pvon_mises, args = args, i = i, draws = draws
   )
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_asym_laplace <- function(i, draws, ...) {
@@ -444,19 +438,19 @@ log_lik_asym_laplace <- function(i, draws, ...) {
                sigma = get_dpar(draws, "sigma", i = i),
                quantile = get_dpar(draws, "quantile", i = i))
   out <- log_lik_censor(dist = "asym_laplace", args = args, 
-                       i = i, data = draws$data)
+                       i = i, draws = draws)
   out <- log_lik_truncate(out, cdf = pvon_mises, args = args,
-                         i = i, data = draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+                         i = i, draws = draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_hurdle_poisson <- function(i, draws, data = data.frame()) {
   hu <- get_dpar(draws, "hu", i)
   lambda <- get_dpar(draws, "mu", i)
   args <- nlist(lambda, hu)
-  out <- log_lik_censor("hurdle_poisson", args, i, draws$data)
-  out <- log_lik_truncate(out, phurdle_poisson, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("hurdle_poisson", args, i, draws)
+  out <- log_lik_truncate(out, phurdle_poisson, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_hurdle_negbinomial <- function(i, draws, data = data.frame()) {
@@ -464,9 +458,9 @@ log_lik_hurdle_negbinomial <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   shape <- get_dpar(draws, "shape", i = i)
   args <- nlist(mu, shape, hu)
-  out <- log_lik_censor("hurdle_negbinomial", args, i, draws$data)
-  out <- log_lik_truncate(out, phurdle_negbinomial, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("hurdle_negbinomial", args, i, draws)
+  out <- log_lik_truncate(out, phurdle_negbinomial, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_hurdle_gamma <- function(i, draws, data = data.frame()) {
@@ -474,9 +468,9 @@ log_lik_hurdle_gamma <- function(i, draws, data = data.frame()) {
   shape <- get_dpar(draws, "shape", i = i)
   scale <- get_dpar(draws, "mu", i) / shape
   args <- nlist(shape, scale, hu)
-  out <- log_lik_censor("hurdle_gamma", args, i, draws$data)
-  out <- log_lik_truncate(out, phurdle_gamma, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("hurdle_gamma", args, i, draws)
+  out <- log_lik_truncate(out, phurdle_gamma, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_hurdle_lognormal <- function(i, draws, data = data.frame()) {
@@ -484,18 +478,18 @@ log_lik_hurdle_lognormal <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   sigma <- get_dpar(draws, "sigma", i = i)
   args <- nlist(mu, sigma, hu)
-  out <- log_lik_censor("hurdle_lognormal", args, i, draws$data)
-  out <- log_lik_truncate(out, phurdle_lognormal, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("hurdle_lognormal", args, i, draws)
+  out <- log_lik_truncate(out, phurdle_lognormal, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_zero_inflated_poisson <- function(i, draws, data = data.frame()) {
   zi <- get_dpar(draws, "zi", i)
   lambda <- get_dpar(draws, "mu", i)
   args <- nlist(lambda, zi)
-  out <- log_lik_censor("zero_inflated_poisson", args, i, draws$data)
-  out <- log_lik_truncate(out, pzero_inflated_poisson, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("zero_inflated_poisson", args, i, draws)
+  out <- log_lik_truncate(out, pzero_inflated_poisson, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_zero_inflated_negbinomial <- function(i, draws, data = data.frame()) {
@@ -503,9 +497,9 @@ log_lik_zero_inflated_negbinomial <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   shape <- get_dpar(draws, "shape", i = i)
   args <- nlist(mu, shape, zi)
-  out <- log_lik_censor("zero_inflated_negbinomial", args, i, draws$data)
-  out <- log_lik_truncate(out, pzero_inflated_negbinomial, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("zero_inflated_negbinomial", args, i, draws)
+  out <- log_lik_truncate(out, pzero_inflated_negbinomial, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_zero_inflated_binomial <- function(i, draws, data = data.frame()) {
@@ -513,9 +507,9 @@ log_lik_zero_inflated_binomial <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i) 
   zi <- get_dpar(draws, "zi", i)
   args <- list(size = trials, prob = mu, zi)
-  out <- log_lik_censor("zero_inflated_binomial", args, i, draws$data)
-  out <- log_lik_truncate(out, pzero_inflated_binomial, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("zero_inflated_binomial", args, i, draws)
+  out <- log_lik_truncate(out, pzero_inflated_binomial, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_zero_inflated_beta <- function(i, draws, data = data.frame()) {
@@ -523,9 +517,9 @@ log_lik_zero_inflated_beta <- function(i, draws, data = data.frame()) {
   mu <- get_dpar(draws, "mu", i)
   phi <- get_dpar(draws, "phi", i)
   args <- nlist(shape1 = mu * phi, shape2 = (1 - mu) * phi, zi)
-  out <- log_lik_censor("zero_inflated_beta", args, i, draws$data)
-  out <- log_lik_truncate(out, pzero_inflated_beta, args, i, draws$data)
-  log_lik_weight(out, i = i, data = draws$data)
+  out <- log_lik_censor("zero_inflated_beta", args, i, draws)
+  out <- log_lik_truncate(out, pzero_inflated_beta, args, i, draws)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_zero_one_inflated_beta <- function(i, draws, data = data.frame()) {
@@ -541,7 +535,7 @@ log_lik_zero_one_inflated_beta <- function(i, draws, data = data.frame()) {
     out <- dbinom(0, size = 1, prob = zoi, log = TRUE) + 
       do_call(dbeta, c(draws$data$Y[i], args, log = TRUE))
   }
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_categorical <- function(i, draws, data = data.frame()) {
@@ -549,7 +543,7 @@ log_lik_categorical <- function(i, draws, data = data.frame()) {
   eta <- sapply(names(draws$dpars), get_dpar, draws = draws, i = i)
   eta <- insert_refcat(eta, family = draws$family)
   out <- dcategorical(draws$data$Y[i], eta = eta, log = TRUE)
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_multinomial <- function(i, draws, data = data.frame()) {
@@ -557,7 +551,7 @@ log_lik_multinomial <- function(i, draws, data = data.frame()) {
   eta <- sapply(names(draws$dpars), get_dpar, draws = draws, i = i)
   eta <- insert_refcat(eta, family = draws$family)
   out <- dmultinomial(draws$data$Y[i, ], eta = eta, log = TRUE)
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_dirichlet <- function(i, draws, data = data.frame()) {
@@ -569,12 +563,14 @@ log_lik_dirichlet <- function(i, draws, data = data.frame()) {
   cats <- seq_len(draws$data$ncat)
   alpha <- dcategorical(cats, eta = eta) * phi
   out <- ddirichlet(draws$data$Y[i, ], alpha = alpha, log = TRUE)
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_cumulative <- function(i, draws, data = data.frame()) {
   ncat <- draws$data$ncat
-  eta <- get_dpar(draws, "disc", i = i) * get_dpar(draws, "mu", i = i)
+  disc <- get_dpar(draws, "disc", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  eta <- disc * (draws$thres - mu)
   y <- draws$data$Y[i]
   if (y == 1) { 
     out <- log(ilink(eta[, 1], draws$family$link))
@@ -586,12 +582,14 @@ log_lik_cumulative <- function(i, draws, data = data.frame()) {
         ilink(eta[, y - 1], draws$family$link)
     )
   }
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_sratio <- function(i, draws, data = data.frame()) {
   ncat <- draws$data$ncat
-  eta <- get_dpar(draws, "disc", i = i) * get_dpar(draws, "mu", i = i)
+  disc <- get_dpar(draws, "disc", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  eta <- disc * (draws$thres - mu)
   y <- draws$data$Y[i]
   q <- sapply(seq_len(min(y, ncat - 1)), 
     function(k) 1 - ilink(eta[, k], draws$family$link)
@@ -605,12 +603,14 @@ log_lik_sratio <- function(i, draws, data = data.frame()) {
   } else {
     out <- log(1 - q[, y]) + rowSums(log(q[, 1:(y - 1)]))
   }
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_cratio <- function(i, draws, data = data.frame()) {
   ncat <- draws$data$ncat
-  eta <- get_dpar(draws, "disc", i = i) * get_dpar(draws, "mu", i = i)
+  disc <- get_dpar(draws, "disc", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  eta <- disc * (mu - draws$thres)
   y <- draws$data$Y[i]
   q <- sapply(seq_len(min(y, ncat - 1)), 
     function(k) ilink(eta[, k], draws$family$link)
@@ -624,12 +624,14 @@ log_lik_cratio <- function(i, draws, data = data.frame()) {
   } else {
     out <- log(1 - q[, y]) + rowSums(log(q[, 1:(y - 1)]))
   }
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_acat <- function(i, draws, data = data.frame()) {
   ncat <- draws$data$ncat
-  eta <- get_dpar(draws, "disc", i = i) * get_dpar(draws, "mu", i = i)
+  disc <- get_dpar(draws, "disc", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  eta <- disc * (mu - draws$thres)
   y <- draws$data$Y[i]
   if (draws$family$link == "logit") { # more efficient calculation 
     q <- sapply(1:(ncat - 1), function(k) eta[, k])
@@ -652,7 +654,7 @@ log_lik_acat <- function(i, draws, data = data.frame()) {
     p[, ncat] <- apply(q[, 1:(ncat - 1)], 1, prod)
     out <- log(p[, y]) - log(apply(p, 1, sum))
   }
-  log_lik_weight(out, i = i, data = draws$data)
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 log_lik_custom <- function(i, draws, data = data.frame()) {
@@ -679,72 +681,69 @@ log_lik_mixture <- function(i, draws, data = data.frame()) {
   } else {
     out <- log(rowSums(out))
   }
-  out
+  log_lik_weight(out, i = i, draws = draws)
 }
 
 # ----------- log_lik helper-functions -----------
-
-log_lik_censor <- function(dist, args, i, data) {
-  # compute (possibly censored) log_lik values
-  # Args:
-  #   dist: name of a distribution for which the functions
-  #         d<dist> (pdf) and p<dist> (cdf) are available
-  #   args: additional arguments passed to pdf and cdf
-  #   data: data initially passed to Stan
-  # Returns:
-  #   vector of log_lik values
+# compute (possibly censored) log_lik values
+# @param dist name of a distribution for which the functions
+#   d<dist> (pdf) and p<dist> (cdf) are available
+# @param args additional arguments passed to pdf and cdf
+# @param draws a brmsdraws object
+# @return vector of log_lik values
+log_lik_censor <- function(dist, args, i, draws) {
   pdf <- get(paste0("d", dist), mode = "function")
   cdf <- get(paste0("p", dist), mode = "function")
-  if (is.null(data$cens) || data$cens[i] == 0) {
-    do_call(pdf, c(data$Y[i], args, log = TRUE))
-  } else if (data$cens[i] == 1) {
-    do_call(cdf, c(data$Y[i], args, lower.tail = FALSE, log.p = TRUE))
-  } else if (data$cens[i] == -1) {
-    do_call(cdf, c(data$Y[i], args, log.p = TRUE))
-  } else if (data$cens[i] == 2) {
-    log(do_call(cdf, c(data$rcens[i], args)) - 
-          do_call(cdf, c(data$Y[i], args)))
+  y <- draws$data$Y[i]
+  cens <- draws$data$cens[i]
+  if (is.null(cens) || cens == 0) {
+    x <- do_call(pdf, c(y, args, log = TRUE))
+  } else if (cens == 1) {
+    x <- do_call(cdf, c(y, args, lower.tail = FALSE, log.p = TRUE))
+  } else if (cens == -1) {
+    x <- do_call(cdf, c(y, args, log.p = TRUE))
+  } else if (cens == 2) {
+    rcens <- draws$data$rcens[i]
+    x <- log(do_call(cdf, c(rcens, args)) - do_call(cdf, c(y, args)))
   }
+  x
 }
 
-log_lik_truncate <- function(x, cdf, args, i, data) {
-  # adjust log_lik in truncated models
-  # Args:
-  #   x: vector of log_lik values
-  #   cdf: a cumulative distribution function 
-  #   args: arguments passed to cdf
-  #   i: observation number
-  #   data: data initally passed to Stan
-  # Returns:
-  #   vector of log_lik values
-  if (!(is.null(data$lb) && is.null(data$ub))) {
-    lb <- if (is.null(data$lb)) -Inf else data$lb[i]
-    ub <- if (is.null(data$ub)) -Inf else data$ub[i]
+# adjust log_lik in truncated models
+# @param x vector of log_lik values
+# @param cdf a cumulative distribution function 
+# @param args arguments passed to cdf
+# @param i observation number
+# @param draws a brmsdraws object
+# @return vector of log_lik values
+log_lik_truncate <- function(x, cdf, args, i, draws) {
+  lb <- draws$data$lb[i]
+  ub <- draws$data$ub[i]
+  if (!(is.null(lb) && is.null(ub))) {
+    if (is.null(lb)) lb <- -Inf
+    if (is.null(ub)) ub <- Inf
     x - log(do_call(cdf, c(ub, args)) - do_call(cdf, c(lb, args)))
-  } else {
-    x
   }
+  x
 }
 
-log_lik_weight <- function(x, i, data) {
-  # weight log_lik values according to defined weights
-  # Args:
-  #   x: vector of log_lik values
-  #   i: observation number
-  #   data: data initially passed to Stan
-  # Returns:
-  #   vector of log_lik values
-  if ("weights" %in% names(data)) {
-    x * data$weights[i]
-  } else {
-    x
+# weight log_lik values according to defined weights
+# @param x vector of log_lik values
+# @param i observation number
+# @param draws a brmsdraws object
+# @return vector of log_lik values
+log_lik_weight <- function(x, i, draws) {
+  weight <- draws$data$weights[i]
+  if (!is.null(weight)) {
+    x <- x * weight
   }
+  x
 }
 
+# after some discussion with Aki Vehtari and Daniel Simpson,
+# I disallowed computation of log-likelihood values for some models
+# until pointwise solutions are implemented
 stop_no_pw <- function() {
-  # after some discussion with Aki Vehtari and Daniel Simpson,
-  # I disallowed computation of log-likelihood values for some models
-  # until pointwise solutions are implemented
   stop2("Cannot yet compute pointwise log-likelihood for this model ",
         "because the observations are not conditionally independent.")
 }
