@@ -1,10 +1,9 @@
+# type-stable indexing of vector and matrix type objects
+# @param x an R object typically a vector or matrix
+# @param i optional index; if NULL, x is returned unchanged
+# @param row indicating if rows or cols should be indexed
+#   only relevant if x has two or three dimensions
 p <- function(x, i = NULL, row = TRUE) {
-  # flexible indexing of vector and matrix type objects
-  # Args:
-  #   x: an R object typically a vector or matrix
-  #   i: optional index; if NULL, x is returned unchanged
-  #   row: indicating if rows or cols should be indexed
-  #        only relevant if x has two or three dimensions
   if (isTRUE(length(dim(x)) > 3L)) {
     stop2("'p' can only handle objects up to 3 dimensions.")
   }
@@ -28,12 +27,11 @@ p <- function(x, i = NULL, row = TRUE) {
   out
 }
 
+# extract parts of an object with selective dropping of dimensions
+# @param x,...,drop same as in x[..., drop]
+# @drop_dim: Optional numeric or logical vector controlling 
+#   which dimensions to drop. Will overwrite argument 'drop'.
 extract <- function(x, ..., drop = FALSE, drop_dim = NULL) {
-  # extract parts of an object with selective dropping of dimensions
-  # Args:
-  #   x, ..., drop: same as in x[..., drop]
-  #   drop_dim: Optional numeric or logical vector controlling 
-  #     which dimensions to drop. Will overwrite argument 'drop'.
   if (!length(dim(x))) {
     return(x[...])
   }
@@ -62,11 +60,10 @@ extract <- function(x, ..., drop = FALSE, drop_dim = NULL) {
   out
 }
 
+# savely extract columns without dropping other dimensions
+# @param x an array
+# @param i colum index
 extract_col <- function(x, i) {
-  # savely extract columns without dropping other dimensions
-  # Args:
-  #   x: an array
-  #   i: colum index
   ldim <- length(dim(x))
   if (ldim < 2L) {
     return(x)
@@ -101,8 +98,8 @@ seq_dim <- function(x, dim) {
   out
 }
 
+# match rows in x with rows in y
 match_rows <- function(x, y, ...) {
-  # match rows in x with rows in y
   x <- as.data.frame(x)
   y <- as.data.frame(y)
   x <- do_call("paste", c(x, sep = "\r"))
@@ -110,8 +107,8 @@ match_rows <- function(x, y, ...) {
   match(x, y, ...)
 }
 
+# find elements of x matching subelements passed via ls and ...
 find_elements <- function(x, ..., ls = list(), fun = '%in%') {
-  # find elements of x matching subelements passed via ls and ...
   x <- as.list(x)
   if (!length(x)) {
     return(logical(0))
@@ -131,9 +128,9 @@ find_elements <- function(x, ..., ls = list(), fun = '%in%') {
   out
 }
 
+# find rows of x matching columns passed via ls and ...
+# similar to 'find_elements' but for matrix like objects
 find_rows <- function(x, ..., ls = list(), fun = '%in%') {
-  # find rows of x matching columns passed via ls and ...
-  # similar to 'find_elements' but for matrix like objects
   x <- as.data.frame(x)
   if (!nrow(x)) {
     return(logical(0))
@@ -152,19 +149,17 @@ find_rows <- function(x, ..., ls = list(), fun = '%in%') {
   out
 }
 
+# subset x using arguments passed via ls and ...
 subset2 <- function(x, ..., ls = list(), fun = '%in%') {
-  # subset x using arguments passed via ls and ...
   x[find_rows(x, ..., ls = ls, fun = fun), , drop = FALSE]
 }
 
+# convert array to list of elements with reduced dimension
+# @param x an arrary of dimension d
+# @return a list of arrays of dimension d-1
 array2list <- function(x) {
-  # convert array to list of elements with reduced dimension
-  # Args: 
-  #   x: an arrary of dimension d
-  # Returns: 
-  #   A list of arrays of dimension d-1
   if (is.null(dim(x))) {
-    stop("Argument 'x' has no dimension.")
+    return(as.list(x))
   }
   ndim <- length(dim(x))
   out <- list(length = dim(x)[ndim])
@@ -180,35 +175,33 @@ array2list <- function(x) {
   out
 }
 
+# move elements to the start of a named object
 move2start <- function(x, first) {
-  # move elements to the start of a named object
   x[c(first, setdiff(names(x), first))]
 }
 
+# wrapper around replicate but without simplifying
 repl <- function(expr, n) {
-  # wrapper around replicate but without simplifying
   replicate(n, expr, simplify = FALSE)
 }
 
+# find the first element in A that is greater than target
+# @param A a matrix
+# @param target a vector of length nrow(A)
+# @param i column of A being checked first
+# @return a vector of the same length as target containing the 
+#   column ids where A[,i] was first greater than target
 first_greater <- function(A, target, i = 1) {
-  # find the first element in A that is greater than target
-  # Args: 
-  #   A: a matrix
-  #   target: a vector of length nrow(A)
-  #   i: column of A being checked first
-  # Returns: 
-  #   A vector of the same length as target containing the column ids 
-  #   where A[,i] was first greater than target
   ifelse(target <= A[, i] | ncol(A) == i, i, first_greater(A, target, i + 1))
 }
 
+# check if an object is NULL
 isNULL <- function(x) {
-  # check if an object is NULL
   is.null(x) || ifelse(is.vector(x), all(sapply(x, is.null)), FALSE)
 }
 
+# recursively removes NULL entries from an object
 rmNULL <- function(x, recursive = TRUE) {
-  # recursively removes NULL entries from an object
   x <- Filter(Negate(isNULL), x)
   if (recursive) {
     x <- lapply(x, function(x) if (is.list(x)) rmNULL(x) else x)
@@ -216,8 +209,8 @@ rmNULL <- function(x, recursive = TRUE) {
   x
 }
 
+# find the first argument that is not NULL
 first_not_null <- function(...) {
-  # find the first argument that is not NULL
   dots <- list(...)
   out <- NULL
   i <- 1L
@@ -238,13 +231,13 @@ is_equal <- function(x, y, ...) {
   isTRUE(all.equal(x, y, ...))
 }
 
+# check if x behaves like a factor in design matrices
 is_like_factor <- function(x) {
-  # check if x behaves like a factor in design matrices
   is.factor(x) || is.character(x) || is.logical(x)
 }
 
+# as.factor but allows to pass levels
 as_factor <- function(x, levels = NULL) {
-  # similar to as.factor but allows to pass levels
   if (is.null(levels)) {
     out <- as.factor(x)
   } else {
@@ -253,8 +246,8 @@ as_factor <- function(x, levels = NULL) {
   out
 }
 
+# coerce 'x' to a single logical value
 as_one_logical <- function(x, allow_na = FALSE) {
-  # coerce 'x' to TRUE or FALSE if possible
   s <- substitute(x)
   x <- as.logical(x)
   if (length(x) != 1L || anyNA(x) && !allow_na) {
@@ -264,8 +257,8 @@ as_one_logical <- function(x, allow_na = FALSE) {
   x
 }
 
+# coerce 'x' to a single numeric value
 as_one_numeric <- function(x, allow_na = FALSE) {
-  # coerce 'x' to a signle number value
   s <- substitute(x)
   x <- SW(as.numeric(x))
   if (length(x) != 1L || anyNA(x) && !allow_na) {
@@ -275,8 +268,8 @@ as_one_numeric <- function(x, allow_na = FALSE) {
   x
 }
 
+# coerce 'x' to a single character string
 as_one_character <- function(x, allow_na = FALSE) {
-  # coerce 'x' to a single character string
   s <- substitute(x)
   x <- as.character(x)
   if (length(x) != 1L || anyNA(x) && !allow_na) {
@@ -294,11 +287,12 @@ has_cols <- function(x) {
   isTRUE(ncol(x) > 0L)
 }
 
+# expand arguments to the same length
+# @param ... arguments to expand
+# @param length optional expansion length
+#   otherwise taken to be the largest supplied length
+# @return a data.frame with one variable per element in '...'
 expand <- function(..., dots = list(), length = NULL) {
-  # expand arguments of be of the same length
-  # Args:
-  #   ...: arguments to expand
-  #   length: optional expansion length
   dots <- c(dots, list(...))
   max_dim <- NULL
   if (is.null(length)) {
@@ -310,23 +304,22 @@ expand <- function(..., dots = list(), length = NULL) {
   structure(out, max_dim = max_dim)
 }
 
+# structure but ignore NULL
 structure_not_null <- function(.Data, ...) {
-  # structure but ignore NULL
   if (!is.null(.Data)) {
     .Data <- structure(.Data, ...)
   }
   .Data
 }
 
+# remove specified attributes
 rm_attr <- function(x, attr) {
-  # remove certain attributes
   attributes(x)[attr] <- NULL
   x
 }
 
+# unidemnsional subsetting while keeping attributes
 subset_keep_attr <- function(x, y) {
-  # take a subset of vector, list, etc.
-  # while keeping all attributes except for names
   att <- attributes(x)
   x <- x[y]
   att$names <- names(x)
@@ -334,8 +327,8 @@ subset_keep_attr <- function(x, y) {
   x
 }
 
+# check if x is a whole number (integer)
 is_wholenumber <- function(x, tol = .Machine$double.eps) {  
-  # check if x is a whole number (integer)
   if (is.numeric(x)) {
     out <- abs(x - round(x)) < tol
   } else {
@@ -345,26 +338,36 @@ is_wholenumber <- function(x, tol = .Machine$double.eps) {
   out
 }
 
+# helper function to check symmetry of a matrix
 is_symmetric <- function(x, tol = sqrt(.Machine$double.eps)) {
-  # helper function to check symmetry of a matrix
   isSymmetric(x, tol = tol, check.attributes = FALSE)
 }
 
+# unlist lapply output
 ulapply <- function(X, FUN, ..., recursive = TRUE, use.names = TRUE) {
-  # short for unlist(lapply())
   unlist(lapply(X, FUN, ...), recursive, use.names)
 }
 
+# rbind lapply output
+rblapply <- function(X, FUN, ...) {
+  do_call(rbind, lapply(X, FUN, ...))
+}
+
+# cbind lapply output
+cblapply <- function(X, FUN, ...) {
+  do_call(cbind, lapply(X, FUN, ...))
+}
+
+# find variables in a character string or expression
 all_vars <- function(expr, ...) {
-  # like all.vars but can handle characters
   if (is.character(expr)) {
     expr <- parse(text = expr)
   }
   all.vars(expr, ...)
 }
 
+# append list(...) to x
 lc <- function(x, ...) {
-  # append list(...) to x
   dots <- rmNULL(list(...), recursive = FALSE)
   c(x, dots)
 }
@@ -378,7 +381,6 @@ lc <- function(x, ...) {
 }
 
 collapse <- function(..., sep = "") {
-  # wrapper for paste with collapse = ""
   paste(..., sep = sep, collapse = "")
 }
 
@@ -386,25 +388,32 @@ collapse_comma <- function(...) {
   paste0("'", ..., "'", collapse = ", ")
 }
 
+# add characters to an existing string
 'str_add<-' <- function(x, start = FALSE, value) {
-  # add characters to an existing string
   if (start) paste0(value, x) else paste0(x, value)
 }
 
+# add list of characters to an existing list
+'str_add_list<-' <- function(x, start = FALSE, value) {
+  stopifnot(is.list(x), is.list(value))
+  out <- if (start) list(value, x) else list(x, value)
+  collapse_lists(ls = out)
+}
+
+# type-stable if clause for strings with default else output
 str_if <- function(cond, yes, no = "") {
-  # type-stable if clause for strings with default else output
   cond <- as_one_logical(cond)
   if (cond) as.character(yes) else as.character(no)
 }
 
+# select elements which match a regex pattern
 str_subset <- function(x, pattern, ...) {
-  # select elements which match a regex pattern
   x[grepl(pattern, x, ...)]
 }
 
+# similar to glue::glue but specialized for generating Stan code
 glue <- function(..., sep = "", collapse = NULL, envir = parent.frame(), 
                  open = "{", close = "}", na = "NA") {
-  # similar to glue::glue but specialized for generating Stan code
   dots <- list(...)
   dots <- dots[lengths(dots) > 0L]
   args <- list(
@@ -428,14 +437,14 @@ zero_length_transformer <- function(text, envir) {
   out
 }
 
+# collapse strings evaluated with glue
 cglue <- function(..., envir = parent.frame()) {
-  # collapse strings evaluated with glue
   glue(..., envir = envir, collapse = "")
 }
 
+# like stats:::na.omit.data.frame but allows to ignore variables
+# keeps NAs in variables with attribute keep_na = TRUE
 na.omit2 <- function(object, ...) {
-  # like stats:::na.omit.data.frame but allows to ignore variables
-  # keeps NAs in variables with attribute keep_na = TRUE
   stopifnot(is.data.frame(object))
   omit <- logical(nrow(object))
   for (j in seq_along(object)) {
@@ -480,18 +489,16 @@ require_package <- function(package, version = NULL) {
   invisible(TRUE)
 }
 
+# rename specified patterns in a character vector
+# @param x a character vector to be renamed
+# @param pattern the regular expressions in x to be replaced
+# @param replacement the replacements
+# @param fixed same as for 'gsub'
+# @param check_dup: logical; check for duplications in x after renaming
+# @param ... passed to 'gsub'
+# @return renamed character vector of the same length as x
 rename <- function(x, pattern = NULL, replacement = NULL, 
                    fixed = TRUE, check_dup = FALSE, ...) {
-  # rename certain patterns in a character vector
-  # Args:
-  #   x: a character vector to be renamed
-  #   pattern: the regular expressions in x to be replaced
-  #   replacement: the replacements
-  #   fixed: same as for sub, grepl etc
-  #   check_dup: logical; check for duplications in x after renaming
-  #   ...: passed to gsub
-  # Returns: 
-  #   renamed character vector of the same length as x
   pattern <- as.character(pattern)
   replacement <- as.character(replacement)
   if (!length(pattern) && !length(replacement)) {
@@ -523,23 +530,21 @@ rename <- function(x, pattern = NULL, replacement = NULL,
   out
 }
 
+# collapse strings having the same name in different lists
+# @param ... named lists
+# @param ls a list of named lists
+# @param a named list containing the collapsed strings
 collapse_lists <- function(..., ls = list()) {
-  # collapse strings having the same name in different lists
-  # Args:
-  #  ls: a list of named lists
-  # Returns:
-  #  a named list containg the collapsed strings
   ls <- c(list(...), ls)
   elements <- unique(unlist(lapply(ls, names)))
-  out <- do_call(mapply, 
-    c(FUN = collapse, lapply(ls, "[", elements), SIMPLIFY = FALSE)
-  )
+  args <- c(FUN = collapse, lapply(ls, "[", elements), SIMPLIFY = FALSE)
+  out <- do_call(mapply, args)
   names(out) <- elements
   out
 }
 
+# create a named list using object names
 nlist <- function(...) {
-  # create a named list using object names
   m <- match.call()
   dots <- list(...)
   no_names <- is.null(names(dots))
@@ -554,11 +559,10 @@ nlist <- function(...) {
   dots
 }
 
+# initialize a named list
+# @param names names of the elements
+# @param values optional values of the elements
 named_list <- function(names, values = NULL) {
-  # initialize a named list
-  # Args:
-  #   names: names of the elements
-  #   values: values of the elements
   if (!is.null(values)) {
     if (length(values) <= 1L) {
       values <- replicate(length(names), values)
@@ -622,12 +626,11 @@ empty_data_frame <- function() {
   as.data.frame(matrix(nrow = 0, ncol = 0))
 }
 
+# replace elements in x with elements in value
+# @param x named list-like object
+# @param value another named list-like object
+# @param dont_replace names of elements that cannot be replaced
 'replace_args<-' <- function(x, dont_replace = NULL, value) {
-  # replace elements in x with elements in value
-  # Args:
-  #   x: named list like object
-  #   value: another named list like object
-  #   dont_replace names of elements that cannot be replaced
   value_name <- deparse_combine(substitute(value), max_char = 100L)
   value <- as.list(value)
   if (length(value) && is.null(names(value))) {
@@ -642,16 +645,16 @@ empty_data_frame <- function() {
   x
 }
 
+# deparse 'x' if it is no string
 deparse_no_string <- function(x) {
-  # deparse x if it is no string
   if (!is.character(x)) {
     x <- deparse(x)
   }
   x
 }
 
+# combine deparse lines into one string
 deparse_combine <- function(x, max_char = NULL) {
-  # combine deparse lines into one string
   out <- collapse(deparse(x))
   if (isTRUE(max_char > 0)) {
     out <- substr(out, 1L, max_char)
@@ -659,23 +662,22 @@ deparse_combine <- function(x, max_char = NULL) {
   out
 }
 
+# like 'eval' but parses characters before evaluation
 eval2 <- function(expr, envir = parent.frame(), ...) {
-  # like eval() but parses characters before evaluation
   if (is.character(expr)) {
     expr <- parse(text = expr)
   }
   eval(expr, envir, ...)
 }
 
+# evaluate an expression without printing output or messages
+# @param expr expression to be evaluated
+# @param type type of output to be suppressed (see ?sink)
+# @param try wrap evaluation of expr in 'try' and 
+#   not suppress outputs if evaluation fails?
+# @param silent actually evaluate silently?
 eval_silent <- function(expr, type = "output", try = FALSE, 
                         silent = TRUE, ...) {
-  # evaluate an expression without printing output or messages
-  # Args:
-  #   expr: expression to be evaluated
-  #   type: type of output to be suppressed (see ?sink)
-  #   try: wrap evaluation of expr in 'try' and 
-  #     not suppress outputs if evaluation fails?
-  #   silent: actually evaluate silently?
   try <- as_one_logical(try)
   silent <- as_one_logical(silent)
   type <- match.arg(type, c("output", "message"))
@@ -699,29 +701,18 @@ eval_silent <- function(expr, type = "output", try = FALSE,
   out
 }
 
-eval_NA <- function(expr, ...) {
-  # evaluate an expression for all variables set to NA
-  if (is.character(expr)) {
-    expr <- parse(text = expr)
-  }
-  data <- named_list(all.vars(expr), NA_real_)
-  eval(expr, envir = data, ...)
-}
-
+# find the name that 'x' had in a specific environment
 substitute_name <- function(x, envir = parent.frame(), nchar = 50) {
-  # find the name that x had in a specific environment
   out <- substitute(x)
   out <- eval2(paste0("substitute(", out, ")"), envir = envir)
   substr(collapse(deparse(out)), 1, nchar)
 }
 
+# recursive sorting of dependencies
+# @param x named list of dependencies per element
+# @param sorted already sorted element names
+# @return a vector of sorted element names
 sort_dependencies <- function(x, sorted = NULL) {
-  # recursive sorting of dependencies
-  # Args:
-  #   x: named list of dependencies per element
-  #   sorted: already sorted element names
-  # Returns:
-  #   a vector of sorted element names
   if (!length(x)) {
     return(NULL)
   }
@@ -747,11 +738,10 @@ warning2 <- function(...) {
   warning(..., call. = FALSE)
 }
 
+# get first occurrence of 'x' in '...' objects
+# @param x The name of the required element
+# @param ... named R objects that may contain 'x'
 get_arg <- function(x, ...) {
-  # find first occurrence of x in ... objects
-  # Args:
-  #  x: The name of the required element
-  #  ...: R objects that may contain x
   dots <- list(...)
   i <- 1
   out <- NULL
@@ -766,16 +756,15 @@ get_arg <- function(x, ...) {
 }
 
 SW <- function(expr) {
-  # just a short form for suppressWarnings
   base::suppressWarnings(expr)
 }
 
+# get pattern matches in text as vector
+# @param simplify return an atomic vector of matches?
+# @param first only return the first match in each string?
+# @return character vector containing matches
 get_matches <- function(pattern, text, simplify = TRUE, 
                         first = FALSE, ...) {
-  # get pattern matches in text as vector
-  # Args:
-  #   simplify: return an atomic vector of matches?
-  #   first: only return the first match in each string?
   x <- regmatches(text, gregexpr(pattern, text, ...))
   if (first) {
     x <- lapply(x, function(t) if (length(t)) t[1] else t)
@@ -789,9 +778,11 @@ get_matches <- function(pattern, text, simplify = TRUE,
   x
 }
 
+# find matches in the parse tree of an expression
+# @param pattern pattern to be matched
+# @param expr expression to be searched in
+# @return character vector containing matches
 get_matches_expr <- function(pattern, expr, ...) {
-  # loop over the parse tree of 'expr '
-  # and find matches of 'pattern'
   if (is.character(expr)) {
     expr <- parse(text = expr)
   }
@@ -809,16 +800,18 @@ get_matches_expr <- function(pattern, expr, ...) {
   unique(out)
 }
 
+# like 'grepl' but handles (parse trees of) expressions 
 grepl_expr <- function(pattern, expr, ...) {
-  # like base::grepl but handles (parse trees of) expressions 
   as.logical(ulapply(expr, function(e) 
     length(get_matches_expr(pattern, e, ...)) > 0L))
 }
 
+# escape dots in character strings
 escape_dot <- function(x) {
   gsub(".", "\\.", x, fixed = TRUE)
 }
 
+# escape all special characters in character strings
 escape_all <- function(x) {
   special <- c(".", "*", "+", "?", "^", "$", "(", ")", "[", "]", "|")
   for (s in special) {
@@ -827,11 +820,10 @@ escape_all <- function(x) {
   x
 }
 
+# add an underscore to non-empty character strings
+# @param x a character vector
+# @param pos position of the underscore
 usc <- function(x, pos = c("prefix", "suffix")) {
-  # add an underscore to non-empty character strings
-  # Args:
-  #   x: a character vector
-  #   pos: position of the underscore
   pos <- match.arg(pos)
   x <- as.character(x)
   if (!length(x)) x <- ""
@@ -843,8 +835,8 @@ usc <- function(x, pos = c("prefix", "suffix")) {
   x
 }
 
+# round using the largest remainder method
 round_largest_remainder <- function(x) {
-  # round using the largest remainder method
   x <- as.numeric(x)
   total <- round(sum(x))
   out <- floor(x)
@@ -855,11 +847,10 @@ round_largest_remainder <- function(x) {
   out
 }
 
+# add leading and trailing whitespaces
+# @param x object accepted by paste
+# @param nsp number of whitespaces to add
 wsp <- function(x = "", nsp = 1) {
-  # add leading and trailing whitespaces
-  # Args:
-  #   x: object accepted by paste
-  #   nsp: number of whitespaces to add
   sp <- collapse(rep(" ", nsp))
   if (length(x)) {
     out <- ifelse(nzchar(x), paste0(sp, x, sp), sp)
@@ -869,18 +860,17 @@ wsp <- function(x = "", nsp = 1) {
   out
 }
 
+# remove whitespaces from strings
 rm_wsp <- function(x) {
-  # remove whitespaces from strings
   gsub("[ \t\r\n]+", "", x, perl = TRUE)
 }
 
+# limit the number of characters of a vector
+# @param x a character vector
+# @param chars maximum number of characters to show
+# @param lsuffix number of characters to keep at the end of the strings
+# @return possible truncated character vector
 limit_chars <- function(x, chars = NULL, lsuffix = 4) {
-  # limit the number of characters of a vector
-  # Args:
-  #   x: a character vector
-  #   chars: maximum number of characters to show
-  #   lsuffix: number of characters to keep 
-  #            at the end of the strings
   stopifnot(is.character(x))
   if (!is.null(chars)) {
     chars_x <- nchar(x) - lsuffix
@@ -892,14 +882,13 @@ limit_chars <- function(x, chars = NULL, lsuffix = 4) {
   x
 }
 
+# ensure that deprecated arguments still work
+# @param arg input to the new argument
+# @param alias input to the deprecated argument
+# @param default the default value of alias
+# @param warn should a warning be printed if alias is specified?
 use_alias <- function(arg, alias = NULL, default = NULL,
                       warn = TRUE) {
-  # ensure that deprecated arguments still work
-  # Args:
-  #   arg: input to the new argument
-  #   alias: input to the deprecated argument
-  #   default: the default value of alias
-  #   warn: should a warning be printed if alias is specified?
   arg_name <- Reduce(paste, deparse(substitute(arg)))
   alias_name <- Reduce(paste, deparse(substitute(alias)))
   if (!is_equal(alias, default)) {
@@ -927,17 +916,15 @@ warn_deprecated <- function(new, old = as.character(sys.call(sys.parent()))[1])
 }
 
 viridis6 <- function() {
-  # colours taken from the viridis package
   c("#440154", "#414487", "#2A788E", "#22A884", "#7AD151", "#FDE725")
 }
 
 expect_match2 <- function(object, regexp, ..., all = TRUE) {
-  # just testthat::expect_match with fixed = TRUE
   testthat::expect_match(object, regexp, fixed = TRUE, ..., all = all)
 }
 
+# startup messages for brms
 .onAttach <- function(libname, pkgname) {
-  # startup messages for brms
   version <- utils::packageVersion("brms")
   packageStartupMessage(
     "Loading 'brms' package (version ", version, "). Useful instructions\n", 
