@@ -348,9 +348,9 @@ set_prior <- function(prior, class = "b", coef = "", group = "",
   Reduce("+", out)
 }
   
+# validate arguments passed to 'set_prior'
 .set_prior <- function(prior, class, coef, group, resp, 
                        dpar, nlpar, lb, ub, check) {
-  # validate arguments passed to 'set_prior'
   prior <- as_one_character(prior)
   class <- as_one_character(class)
   group <- as_one_character(group)
@@ -425,8 +425,8 @@ prior_ <- function(prior, ...) {
   do_call(set_prior, c(call, seval))
 }
 
+# arguments for which to use standard evaluation
 prior_seval_args <- function() {
-  # arguments for which to use standard evaluation
   c("check")
 }
 
@@ -443,12 +443,15 @@ prior_string <- function(prior, ...) {
 #' may be specified including default priors.
 #' 
 #' @inheritParams brm
-#' @param internal A flag indicating if the names of additional internal parameters 
-#'   should be displayed. Setting priors on these parameters is not recommended
+#' @param internal A flag indicating if the names of additional internal
+#'   parameters should be displayed. Setting priors on these parameters is not
+#'   recommended
+#' @param ... Currently ignored.
 #' 
-#' @return A data.frame with columns \code{prior}, \code{class}, \code{coef}, and \code{group}
-#'   and several rows, each providing information on a parameter (or parameter class) on which
-#'   priors can be specified. The prior column is empty except for internal default priors.
+#' @return A data.frame with columns \code{prior}, \code{class}, \code{coef},
+#'   and \code{group} and several rows, each providing information on a
+#'   parameter (or parameter class) on which priors can be specified. The prior
+#'   column is empty except for internal default priors.
 #'   
 #' @seealso \code{\link{set_prior}}
 #' 
@@ -470,15 +473,14 @@ prior_string <- function(prior, ...) {
 #' 
 #' @export
 get_prior <- function(formula, data, family = gaussian(), autocor = NULL, 
-                      sparse = FALSE, internal = FALSE) {
-  # note that default priors are stored in this function
+                      sparse = NULL, internal = FALSE, ...) {
   if (is.brmsfit(formula)) {
     stop2("Use 'prior_summary' to extract priors from 'brmsfit' objects.")
   }
-  sparse <- as_one_logical(sparse)
   internal <- as_one_logical(internal)
   formula <- validate_formula(
-    formula, data = data, family = family, autocor = autocor
+    formula, data = data, family = family, 
+    autocor = autocor, sparse = sparse
   )
   bterms <- parse_bf(formula)
   data <- update_data(data, bterms = bterms)
@@ -488,7 +490,7 @@ get_prior <- function(formula, data, family = gaussian(), autocor = NULL,
   prior <- empty_brmsprior()
   # priors for distributional parameters
   prior <- prior + prior_predictor(
-    bterms, data = data, sparse = sparse, internal = internal
+    bterms, data = data, internal = internal
   )
   # priors of group-level parameters
   def_scale_prior <- def_scale_prior(bterms, data)
@@ -505,8 +507,9 @@ get_prior <- function(formula, data, family = gaussian(), autocor = NULL,
   structure(prior, class = c("brmsprior", "data.frame"))
 }
 
+# generate priors for predictor terms
+# @return a 'brmsprior' object
 prior_predictor <- function(x, ...) {
-  # generate priors for predictor terms
   UseMethod("prior_predictor")
 }
 
@@ -541,12 +544,27 @@ prior_predictor.mvbrmsterms <- function(x, internal = FALSE, ...) {
   prior
 }
 
-prior_predictor.brmsterms <- function(x, data, sparse = FALSE, ...) {
+prior_predictor.brmsterms <- function(x, data, ...) {
   data <- subset_data(data, x)
   def_scale_prior <- def_scale_prior(x, data)
-  valid_dpars <- valid_dpars(x$family, bterms = x)
+  valid_dpars <- valid_dpars(x)
   prior <- empty_brmsprior()
-  cats <- if (is_ordinal(x)) extract_cat_names(x, data)
+  # priors for mixture models
+  if (is.mixfamily(x$family)) {
+    if (has_joint_theta(x)) {
+      # individual theta parameters should not have a prior in this case
+      theta_dpars <- str_subset(valid_dpars, "^theta[[:digit:]]+")
+      valid_dpars <- setdiff(valid_dpars, theta_dpars)
+      prior <- prior + brmsprior(class = "theta", resp = x$resp)
+    }
+    if (fix_intercepts(x)) {
+      # fixing thresholds across mixture componenents 
+      # requires a single set of priors at the top level
+      stopifnot(is_ordinal(x))
+      prior <- prior + prior_thres(x, def_scale_prior = def_scale_prior)
+    }
+  } 
+  # priors for distributional parameters
   for (dp in valid_dpars) {
     def_dprior <- def_dprior(x, dp, data = data)
     if (!is.null(x$dpars[[dp]])) {
@@ -554,8 +572,7 @@ prior_predictor.brmsterms <- function(x, data, sparse = FALSE, ...) {
       dp_prior <- prior_predictor(
         x$dpars[[dp]], data = data,
         def_scale_prior = def_scale_prior,
-        def_dprior = def_dprior, cats = cats,
-        sparse = sparse
+        def_dprior = def_dprior
       )
     } else if (!is.null(x$fdpars[[dp]])) {
       # parameter is fixed
@@ -566,12 +583,12 @@ prior_predictor.brmsterms <- function(x, data, sparse = FALSE, ...) {
     }
     prior <- prior + dp_prior
   }
+  # priors for non-linear parameters
   for (nlp in names(x$nlpars)) {
     nlp_prior <- prior_predictor(
       x$nlpars[[nlp]], data = data,
       def_scale_prior = def_scale_prior,
-      def_dprior = def_dprior, 
-      sparse = sparse
+      def_dprior = def_dprior
     )
     prior <- prior + nlp_prior
   }
@@ -583,16 +600,6 @@ prior_predictor.brmsterms <- function(x, data, sparse = FALSE, ...) {
       }
     }
   }
-  # prior for the delta parameter for equidistant thresholds
-  if (is_ordinal(x$family) && is_equal(x$family$threshold, "equidistant")) {
-    bound <- ifelse(x$family$family == "cumulative", "<lower=0>", "")
-    prior <- prior + brmsprior(class = "delta", bound = bound, resp = x$resp)
-  }
-  # priors for mixture models
-  dp_classes <- dpar_class(names(c(x$dpars, x$fdpars)))
-  if (is.mixfamily(x$family) && !any(dp_classes == "theta")) {
-    prior <- prior + brmsprior(class = "theta", resp = x$resp)
-  }
   # priors for noise-free response variables
   sdy <- get_sdy(x, data)
   if (!is.null(sdy)) {
@@ -605,64 +612,69 @@ prior_predictor.brmsterms <- function(x, data, sparse = FALSE, ...) {
   prior
 }
 
+# prior for linear predictor termss
 #' @export
-prior_predictor.btl <- function(x, data, ...) {
-  # collect default priors for various kinds of effects
-  # Args:
-  #   spec_intercept: special parameter class for the Intercept?
-  #   def_scale_prior: default prior for SD parameters
-  # Return:
-  #   An object of class brmsprior
-  prior_fe(x, data, ...) +
-    prior_sp(x, data, ...) +
-    prior_cs(x, data, ...) +
-    prior_sm(x, data, ...) + 
-    prior_gp(x, data, ...)
+prior_predictor.btl <- function(x, ...) {
+  prior_fe(x, ...) +
+    prior_thres(x, ...) +
+    prior_sp(x, ...) +
+    prior_cs(x, ...) +
+    prior_sm(x, ...) + 
+    prior_gp(x, ...)
 }
 
+# priors for non-linear predictor terms
 #' @export
-prior_predictor.btnl <- function(x, data, ...) {
-  empty_brmsprior()
+prior_predictor.btnl <- function(x, ...) {
+  # thresholds are required even in non-linear ordinal models
+  prior_thres(x, ...)
 }
 
-prior_fe <- function(bterms, data, sparse = FALSE, 
-                     def_dprior = "", cats = NULL, ...) {
-  # priors for population-level parameters
-  # Returns:
-  #   an object of class brmsprior
+# priors for population-level parameters
+prior_fe <- function(bterms, data, def_dprior = "", ...) {
   prior <- empty_brmsprior()
   fixef <- colnames(data_fe(bterms, data)$X)
   px <- check_prefix(bterms)
-  # shell the intercept get its own prior class?
-  spec_intercept <- !no_center(bterms$fe) && !sparse
-  if (has_intercept(bterms$fe) && spec_intercept) {
-    coefs <- NULL
-    if (is_ordinal(bterms)) {
-      threshold <- bterms$family$threshold
-      if (isTRUE(threshold == "equidistant")) {
-        coefs <- "1"
-      } else {
-        stopifnot(!is.null(cats))
-        coefs <- cats[-length(cats)]
-      }
-    }
-    prior <- prior + brmsprior(
-      c(def_dprior, rep("", length(coefs))), 
-      class = "Intercept", coef = c("", coefs), ls = px
-    )
+  center_X <- stan_center_X(bterms)
+  if (center_X && !is_ordinal(bterms)) {
+    # priors for ordinal thresholds are provided in 'prior_thres'
+    prior <- prior + brmsprior(def_dprior, class = "Intercept", ls = px)
     fixef <- setdiff(fixef, "Intercept")
   }
   if (length(fixef)) {
-    prior <- prior + 
-      brmsprior(class = "b", coef = c("", fixef), ls = px)
+    prior <- prior + brmsprior(class = "b", coef = c("", fixef), ls = px)
   }
   prior
 }
 
+# priors for thresholds of ordinal models
+prior_thres <- function(bterms, def_scale_prior = "", ...) {
+  prior <- empty_brmsprior()
+  if (!is_ordinal(bterms)) {
+    # thresholds only exist in ordinal models
+    return(prior)
+  }
+  if (fix_intercepts(bterms) && !is.mixfamily(bterms$family)) {
+    # fixed thresholds cannot have separate priors
+    return(prior)
+  }
+  px <- check_prefix(bterms)
+  thres <- get_thres(bterms)
+  if (has_equidistant_thres(bterms)) {
+    thres <- thres[1]
+    # prior for the delta parameter for equidistant thresholds
+    bound <- str_if(has_ordered_thres(bterms), "<lower=0>")
+    prior <- prior + brmsprior(class = "delta", bound = bound, ls = px)
+  }
+  prior <- prior + brmsprior(
+    prior = c(def_scale_prior, rep("", length(thres))), 
+    class = "Intercept", coef = c("", thres), ls = px
+  )
+  prior
+}
+
+# priors for special effects parameters
 prior_sp <- function(bterms, data, ...) {
-  # priors for special effects parameters
-  # Returns:
-  #   an object of class brmsprior
   prior <- empty_brmsprior()
   spef <- tidy_spef(bterms, data)
   if (nrow(spef)) {
@@ -680,10 +692,8 @@ prior_sp <- function(bterms, data, ...) {
   prior
 }
 
+# priors for category spcific effects parameters
 prior_cs <- function(bterms, data, ...) {
-  # priors for category spcific effects parameters
-  # Returns:
-  #   an object of class brmsprior
   prior <- empty_brmsprior()
   csef <- colnames(get_model_matrix(bterms$cs, data = data))
   if (length(csef)) {
@@ -694,10 +704,8 @@ prior_cs <- function(bterms, data, ...) {
   prior
 }
 
+# default priors for hyper-parameters of noise-free variables
 prior_Xme <- function(meef, internal = FALSE, ...) {
-  # default priors for hyper-parameters of noise-free variables
-  # Returns:
-  #   an object of class brmsprior
   stopifnot(is.meef_frame(meef))
   prior <- empty_brmsprior()
   if (nrow(meef)) {
@@ -727,12 +735,10 @@ prior_Xme <- function(meef, internal = FALSE, ...) {
   prior
 }
 
+# default priors of gaussian processes
+# @param def_scale_prior: a character string defining 
+#   the default prior SD parameters
 prior_gp <- function(bterms, data, def_scale_prior, ...) {
-  # default priors of gaussian processes
-  # Returns:
-  #   an object of class brmsprior
-  #   def_scale_prior: a character string defining 
-  #     the default prior for random effects SDs
   prior <- empty_brmsprior()
   gpef <- tidy_gpef(bterms, data)
   if (nrow(gpef)) {
@@ -748,12 +754,11 @@ prior_gp <- function(bterms, data, def_scale_prior, ...) {
   prior
 }
 
+# default priors for length-scale parameters of GPs
+# see https://betanalpha.github.io/assets/case_studies/gp_part3/part3.html
+# @param plb prior probability of being lower than minimum length-scale
+# @param pub prior probability of being higher than maximum length-scale
 def_lscale_prior <- function(bterms, data, plb = 0.01, pub = 0.01) {
-  # default priors for length-scale parameters of GPs
-  # see https://betanalpha.github.io/assets/case_studies/gp_part3/part3.html
-  # Args:
-  #   plb: prior probability of being lower than minimum length-scale
-  #   pub: prior probability of being higher than maximum length-scale
   .opt_fun <- function(x, lb, ub) {
     # optimize parameters on the log-scale to make them positive only
     x <- exp(x)
@@ -808,15 +813,12 @@ def_lscale_prior <- function(bterms, data, plb = 0.01, pub = 0.01) {
   unlist(out)
 }
 
+# priors for varying effects parameters
+# @param ranef: a list returned by tidy_ranef
+# @param def_scale_prior a character string defining 
+#   the default prior for SD parameters
+# @param internal: see 'get_prior'
 prior_re <- function(ranef, def_scale_prior, internal = FALSE, ...) {
-  # priors for random effects parameters
-  # Args:
-  #   ranef: a list returned by tidy_ranef
-  #   def_scale_prior: a character string defining the default
-  #     prior for random effects SDs
-  #   internal: see get_prior
-  # Returns:
-  #   an object of class brmsprior
   prior <- empty_brmsprior()
   if (!nrow(ranef)) {
     return(prior)
@@ -871,11 +873,8 @@ prior_re <- function(ranef, def_scale_prior, internal = FALSE, ...) {
   prior
 }
 
+# priors for smooth terms
 prior_sm <- function(bterms, data, def_scale_prior, ...) {
-  # priors for smooth terms
-  # Args:
-  #   def_scale_prior: a character string defining 
-  #     the default prior for smooth SDs
   prior <- empty_brmsprior()
   smef <- tidy_smef(bterms, data)
   if (NROW(smef)) {
@@ -898,21 +897,24 @@ prior_sm <- function(bterms, data, def_scale_prior, ...) {
   prior
 }
 
+# priors for autocor parameters
 prior_autocor <- function(bterms, def_scale_prior) {
-  # priors for autocor parameters
   stopifnot(is.brmsterms(bterms))
   autocor <- bterms$autocor
   resp <- bterms$resp
   cbound <- "<lower=-1,upper=1>"
   prior <- empty_brmsprior()
-  if (get_ar(autocor)) {
-    prior <- prior + brmsprior(class = "ar", resp = resp, bound = cbound)
-  }
-  if (get_ma(autocor)) {
-    prior <- prior + brmsprior(class = "ma", resp = resp, bound = cbound)
-  }
-  if (get_arr(autocor)) {
-    prior <- prior + brmsprior(class = "arr", resp = resp)
+  if (is.cor_arma(autocor)) {
+    if (get_ar(autocor)) {
+      prior <- prior + brmsprior(class = "ar", resp = resp, bound = cbound)
+    }
+    if (get_ma(autocor)) {
+      prior <- prior + brmsprior(class = "ma", resp = resp, bound = cbound)
+    }
+    if (has_latent_residuals(bterms)) {
+      prior <- prior + 
+        brmsprior(def_scale_prior, class = "sderr", resp = resp)
+    }
   }
   if (is.cor_sar(autocor)) {
     if (identical(autocor$type, "lag")) {
@@ -929,15 +931,11 @@ prior_autocor <- function(bterms, def_scale_prior) {
       prior <- prior + brmsprior(class = "car", resp = resp)
     }
   }
-  if (is.cor_bsts(autocor)) {
-    prior <- prior +
-      brmsprior(class = "sigmaLL", prior = def_scale_prior, resp = resp)
-  }
   prior
 }
 
+# default priors for distributional parameters
 def_dprior <- function(x, dpar, data = NULL) {
-  # default priors for distributional parameters
   stopifnot(is.brmsterms(x))
   dpar <- as_one_character(dpar)
   dpar_class <- dpar_class(dpar)
@@ -995,8 +993,8 @@ def_dprior <- function(x, dpar, data = NULL) {
   out
 }
 
+# default priors for scale/SD parameters
 def_scale_prior <- function(x, data, ...) {
-  # ensure that SD parameters have a weakly informative prior by default
   UseMethod("def_scale_prior")
 }
 
@@ -1007,11 +1005,10 @@ def_scale_prior.mvbrmsterms <- function(x, data, ...) {
   out
 }
 
+# @param center Should the prior be centererd around zero?
+#   If FALSE, the prior location is computed based on Y.
 #' @export
 def_scale_prior.brmsterms <- function(x, data, center = TRUE, ...) {
-  # Args:
-  #   center: Should the prior be centererd around zero?
-  #     If FALSE, the prior location is computed based on Y.
   Y <- unname(model.response(model.frame(x$respform, data)))
   prior_location <- 0
   prior_scale <- 10
@@ -1039,15 +1036,13 @@ def_scale_prior.brmsterms <- function(x, data, center = TRUE, ...) {
   paste0("student_t(", sargs("3", prior_location, prior_scale), ")")
 }
 
-check_prior <- function(prior, formula, data, sparse = FALSE, 
+# check priors supplied by the user
+# @inheritParams get_prior
+# @param warn: passed to 'check_prior_content'
+# @return a 'brmsprior' object
+check_prior <- function(prior, formula, data, 
                         sample_prior = c("no", "yes", "only"),
                         warn = FALSE) {
-  # check prior input and amend it if needed
-  # Args:
-  #   same as the respective parameters in brm
-  #   warn: passed to check_prior_content
-  # Returns:
-  #   a data.frame of prior specifications to be used in stan_prior
   sample_prior <- check_sample_prior(sample_prior)
   if (isTRUE(attr(prior, "checked"))) {
     # prior has already been checked; no need to do it twice
@@ -1056,7 +1051,7 @@ check_prior <- function(prior, formula, data, sparse = FALSE,
     return(prior)
   }
   bterms <- parse_bf(formula)
-  all_priors <- get_prior(formula, data, sparse = sparse, internal = TRUE)
+  all_priors <- get_prior(formula, data, internal = TRUE)
   if (is.null(prior)) {
     prior <- all_priors  
   } else if (!is.brmsprior(prior)) {
@@ -1106,11 +1101,10 @@ check_prior <- function(prior, formula, data, sparse = FALSE,
   prior
 }
 
+# try to check if prior distributions are reasonable
+# @param prior A brmsprior object
+# @param warn logical; print boundary warnings?
 check_prior_content <- function(prior, warn = TRUE) {
-  # try to check if prior distributions are reasonable
-  # Args:
-  #  prior: A brmsprior object
-  #  warn: logical; print boundary warnings?
   if (!is.brmsprior(prior)) {
     return(invisible(TRUE))
   }
@@ -1204,8 +1198,9 @@ check_prior_content <- function(prior, warn = TRUE) {
   invisible(TRUE)
 }
 
+# prepare special priors for use in Stan
+# required for priors that are not natively supported by Stan
 check_prior_special <- function(x, ...) {
-  # prepare special priors for use in Stan
   UseMethod("check_prior_special")
 }
 
@@ -1305,15 +1300,12 @@ check_prior_special.btnl <- function(x, prior, ...) {
   prior
 }
 
+# prepare special priors that cannot be passed to Stan as is
+# @param allow_autoscale allow autoscaling by parameter sigma?
+# @return a possibly updated brmsprior object with additional attributes
 #' @export
 check_prior_special.btl <- function(x, prior, data,
                                     allow_autoscale = TRUE, ...) {
-  # prepare special priors that cannot be passed to Stan as is
-  # Args:
-  #   prior: an object of class brmsprior
-  #   allow_autoscale: allow autoscaling using sigma?
-  # Returns:
-  #   a possibly amended brmsprior object with additional attributes
   px <- check_prefix(x)
   if (is_nlpar(x) && no_center(x$fe)) {
     nlp_prior <- subset2(prior, ls = px)
@@ -1401,9 +1393,13 @@ check_prior_special.btl <- function(x, prior, data,
   prior
 }
 
+
+# validate argument 'sample_prior'
 check_sample_prior <- function(sample_prior) {
-  # validate argument 'sample_prior'
   options <- c("no", "yes", "only")
+  if (is.null(sample_prior)) {
+    sample_prior <- "no"
+  }
   if (!is.character(sample_prior)) {
     sample_prior <- as_one_logical(sample_prior)
     sample_prior <- if (sample_prior) "yes" else "no"
@@ -1411,12 +1407,11 @@ check_sample_prior <- function(sample_prior) {
   match.arg(sample_prior, options)
 }
 
+# extract prior boundaries of a parameter
+# @param prior a brmsprior object
+# @param class,coef,group,px passed to 'subset2'
 get_bound <- function(prior, class = "b", coef = "", 
                       group = "", px = list()) {
-  # extract prior boundaries of a parameter
-  # Args:
-  #   prior: object of class brmsprior
-  #   class, coef, group, px: passed to subset2
   stopifnot(is.brmsprior(prior))
   class <- as_one_character(class)
   if (!length(coef)) coef <- ""
@@ -1429,10 +1424,10 @@ get_bound <- function(prior, class = "b", coef = "",
   bound
 }
 
+# create data.frames containing prior information
 brmsprior <- function(prior = "", class = "", coef = "", group = "", 
                       resp = "", dpar = "", nlpar = "", bound = "",
                       ls = list()) {
-  # helper function to create data.frames containing prior information
   if (length(ls)) {
     if (is.null(names(ls))) {
       stop("Argument 'ls' must be named.")
@@ -1456,8 +1451,8 @@ brmsprior <- function(prior = "", class = "", coef = "", group = "",
   out
 }
 
+# brmsprior object with zero rows
 empty_brmsprior <- function() {
-  # define a brmsprior object with zero rows
   char0 <- character(0)
   brmsprior(
     prior = char0, class = char0, coef = char0, 
@@ -1466,10 +1461,9 @@ empty_brmsprior <- function() {
   )
 }
 
+# natural upper and lower bounds for priors
+# @param a named list with elements 'lb and 'ub'
 prior_bounds <- function(prior) {
-  # natural upper and lower bounds for priors
-  # Returns:
-  #   A named list with elements 'lb and 'ub'
   switch(prior,
     lognormal = list(lb = 0, ub = Inf),
     chi_square = list(lb = 0, ub = Inf),
@@ -1489,15 +1483,16 @@ prior_bounds <- function(prior) {
   )
 }
 
+# relevant columns for duplication checks in brmsprior objects
 rcols_prior <- function() {
-  # relevant columns for duplication checks in brmsprior objects
   c("class", "coef", "group", "resp", "dpar", "nlpar")
 }
 
+# upper and lower bounds for parameter classes
+# @param par name of a distributional parameter
+# @param bound optional Stan code of boundaries to extract values from 
+# @return A named list with elements 'lb and 'ub'
 par_bounds <- function(par, bound = "") {
-  # upper and lower bounds for parameter classes
-  # Returns:
-  #   A named list with elements 'lb and 'ub'
   out <- switch(par,
     sigma = list(lb = 0, ub = Inf),
     shape = list(lb = 0, ub = Inf),
@@ -1570,8 +1565,8 @@ print.brmsprior <- function(x, show_df, ...) {
   invisible(x)
 }
 
+# prepare text for print.brmsprior
 .print_prior <- function(x) {
-  # prepare text for print.brmsprior
   group <-  usc(x$group)
   resp <- usc(x$resp)
   dpar <- usc(x$dpar)
@@ -1586,9 +1581,9 @@ print.brmsprior <- function(x, show_df, ...) {
   paste0(bound, x$class, group, resp, dpar, nlpar, coef, tilde, prior)
 }
 
+# combine multiple brmsprior objects into one brmsprior
 #' @export
 c.brmsprior <- function(x, ...) {
-  # combine multiple brmsprior objects into one brmsprior
   if (all(sapply(list(...), is.brmsprior))) {
     out <- do_call(rbind, list(x, ...)) 
   } else {
@@ -1602,9 +1597,9 @@ c.brmsprior <- function(x, ...) {
   c(e1, e2)
 }
 
+# evaluate the dirichlet prior of simplex parameters
+# avoid name clashing with the dirichlet family
 eval_dirichlet <- function(prior) {
-  # evaluate the dirichlet prior of simplex parameters
-  # avoid name clashing with the dirichlet family
   dirichlet <- function(...) {
     out <- as.numeric(c(...))
     if (anyNA(out) || any(out <= 0)) {
@@ -1735,8 +1730,8 @@ horseshoe <- function(df = 1, scale_global = 1, df_global = 1,
   }
   if (!is.null(par_ratio)) {
     par_ratio <- as.numeric(par_ratio)
-    if (!isTRUE(par_ratio > 0 && par_ratio <= 1)) {
-      stop2("Argument 'par_ratio' must be within [0, 1].")
+    if (!isTRUE(par_ratio > 0)) {
+      stop2("Argument 'par_ratio' must be greater 0.")
     }
   }
   autoscale <- as_one_logical(autoscale)
