@@ -1,10 +1,8 @@
+# rename parameters (and possibly change their dimensions) 
+# within the stanfit object to ensure reasonable parameter names
+# @param x a brmsfit obejct
+# @return a brmfit object with adjusted parameter names and dimensions
 rename_pars <- function(x) {
-  # rename parameters (and possibly change their dimensions) 
-  # within the stanfit object to ensure reasonable parameter names
-  # Args:
-  #   x: a brmsfit obejct
-  # Returns:
-  #   a brmfit object with adjusted parameter names and dimensions
   if (!length(x$fit@sim)) {
     return(x) 
   }
@@ -16,18 +14,19 @@ rename_pars <- function(x) {
   change <- c(
     change_effects(bterms, data = data, pars = pars, scode = stancode(x)),
     change_re(x$ranef, pars = pars),
-    change_Xme(meef, pars = pars),
-    change_autocor(bterms, data = data, pars = pars)
+    change_Xme(meef, pars = pars)
   )
   # perform the actual renaming in x$fit@sim
+  x <- save_old_par_order(x)
   x <- do_renaming(x, change)
   x <- compute_quantities(x)
   x <- reorder_pars(x)
   x
 }
 
+# helps in renaming parameters after model fitting
+# @return a list whose elements can be interpreted by do_renaming
 change_effects <- function(x, ...) {
-  # helps in renaming parameters after model fitting
   UseMethod("change_effects")
 }
 
@@ -66,11 +65,11 @@ change_effects.brmsterms <- function(x, ...) {
   out
 }
 
+# helps in renaming parameters of additive predictor terms
+# @param pars vector of all parameter names
+# @param scode stan code of the model
 #' @export
 change_effects.btl <- function(x, data, pars, scode = "", ...) {
-  # helps in renaming various kinds of effects
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   c(change_fe(x, data, pars, scode = scode),
     change_sm(x, data, pars),
     change_cs(x, data, pars),
@@ -78,10 +77,8 @@ change_effects.btl <- function(x, data, pars, scode = "", ...) {
     change_gp(x, data, pars))
 }
 
+# helps in renaming fixed effects parameters
 change_fe <- function(bterms, data, pars, scode = "") {
-  # helps in renaming fixed effects parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   px <- check_prefix(bterms)
   fixef <- colnames(data_fe(bterms, data)$X)
@@ -96,10 +93,8 @@ change_fe <- function(bterms, data, pars, scode = "") {
   out
 }
 
+# helps in renaming special effects parameters
 change_sp <- function(bterms, data, pars) {
-  # helps in renaming special effects parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   spef <- tidy_spef(bterms, data)
   if (!nrow(spef)) return(out)
@@ -123,10 +118,8 @@ change_sp <- function(bterms, data, pars) {
   out
 }
 
+# helps in renaming category specific effects parameters
 change_cs <- function(bterms, data, pars) {
-  # helps in renaming category specific effects parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   csef <- colnames(data_cs(bterms, data)$Xcs)
   if (length(csef)) {
@@ -145,10 +138,9 @@ change_cs <- function(bterms, data, pars) {
   out
 }
 
+# helps in renaming global noise free variables
+# @param meef data.frame returned by 'tidy_meef'
 change_Xme <- function(meef, pars) {
-  # helps in renaming global noise free variables
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   stopifnot(is.meef_frame(meef))
   out <- list()
   levels <- attr(meef, "levels")
@@ -194,8 +186,8 @@ change_Xme <- function(meef, pars) {
   out
 }
 
+# helps in renaming estimated missing values
 change_Ymi <- function(bterms, data, pars, ...) {
-  # helps in renaming estimated missing values
   stopifnot(is.brmsterms(bterms))
   out <- list()
   if (is.formula(bterms$adforms$mi)) {
@@ -212,10 +204,8 @@ change_Ymi <- function(bterms, data, pars, ...) {
   out
 }
 
+# helps in renaming parameters of gaussian processes
 change_gp <- function(bterms, data, pars) {
-  # helps in renaming parameters of gaussian processes
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   p <- usc(combine_prefix(bterms), "prefix")
   gpef <- tidy_gpef(bterms, data)
@@ -262,10 +252,8 @@ change_gp <- function(bterms, data, pars) {
   out
 }
 
+# helps in renaming smoothing term parameters
 change_sm <- function(bterms, data, pars) {
-  # helps in renaming smoothing term parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   smef <- tidy_smef(bterms, data)
   if (NROW(smef)) {
@@ -300,13 +288,9 @@ change_sm <- function(bterms, data, pars) {
   out
 }
 
+# helps in renaming group-level parameters
+# @param ranef: data.frame returned by 'tidy_ranef'
 change_re <- function(ranef, pars) {
-  # helps in renaming group-level parameters
-  # Args:
-  #   ranef: list returned by tidy_ranef
-  #   pars: names of all model parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   if (has_rows(ranef)) {
     for (id in unique(ranef$id)) {
@@ -355,13 +339,9 @@ change_re <- function(ranef, pars) {
   out
 } 
 
+# helps in renaming varying effects parameters per level
+# @param ranef: data.frame returned by 'tidy_ranef'
 change_re_levels <- function(ranef, pars)  {
-  # helps in renaming random effects 'r_.' parameters
-  # Args:
-  #   ranef: output of tidy_ranef
-  #   pars: names of all model parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   for (i in seq_rows(ranef)) {
     r <- ranef[i, ]
@@ -378,38 +358,14 @@ change_re_levels <- function(ranef, pars)  {
   out
 }
 
-change_autocor <- function(bterms, data, pars) {
-  # helps in renaming autocor parameters
-  out <- list()
-  if (is.cor_bsts(bterms$autocor)) {
-    data <- order_data(data, bterms = bterms)
-    if (!is.null(bterms$time$group)) {
-      group <- gsub("[ \t\r\n]", "", get(bterms$time$group, data))
-    } else {
-      group <- rep(1, nrow(data)) 
-    }
-    if (!is.null(bterms$time$time)) {
-      time <- gsub("[ \t\r\n]", "", get(bterms$time$time, data))
-    } else {
-      time <- ulapply(unique(group), function(g) seq_len(sum(group == g)))
-    }
-    loclev_pars <- paste0("loclev[", group, ",", time, "]")
-    lc(out) <- clist(grepl("^loclev\\[", pars), loclev_pars)
-  }
-  out
-}
-
+# helps in renaming prior parameters
+# @param class the class of the parameters
+# @param pars names of all parameters in the model
+# @param names names to replace digits at the end of parameter names
+# @param new_class optional replacement of the orginal class name
+# @param is_vector indicate if the prior parameter is a vector
 change_prior <- function(class, pars, names = NULL, new_class = class,
                          is_vector = FALSE) {
-  # helps in renaming prior parameters
-  # Args: 
-  #   class: the class of the parameters for which prior names should be changed
-  #   pars: names of all parameters in the model
-  #   names: names to replace digits at the end of parameter names
-  #   new_class: replacement of the orginal class name
-  #   is_vector: indicate if the prior parameter is a vector
-  # Return:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   regex <- paste0("^prior_", class, "(_[[:digit:]]+|$|\\[)")
   pos_priors <- which(grepl(regex, pars))
@@ -439,8 +395,8 @@ change_prior <- function(class, pars, names = NULL, new_class = class,
   out
 }
 
+# helper for change_* functions
 clist <- function(pos, fnames, ...) {
-  # helper for change_* functions
   structure(nlist(pos, fnames, ...), class = c("clist", "list"))
 }
 
@@ -448,9 +404,9 @@ is.clist <- function(x) {
   inherits(x, "clist")
 }
 
+# identify if the intercept has to be removed from fixef
+# @return adjusted fixef names
 rm_int_fe <- function(fixef, scode, px = NULL) {
-  # identifies if the intercept has to be removed from fixef
-  # and returns adjusted fixef names
   p <- usc(combine_prefix(px))
   regex <- paste0("(temp", p, "_Intercept)|(vector\\[N\\] loclev", p, ";)")
   if (grepl(regex, scode)) {
@@ -459,14 +415,12 @@ rm_int_fe <- function(fixef, scode, px = NULL) {
   fixef
 }
 
+# compute index names in square brackets for indexing stan parameters
+# @param rownames a vector of row names
+# @param colnames a vector of columns 
+# @param dim the number of output dimensions
+# @return all index pairs of rows and cols
 make_index_names <- function(rownames, colnames = NULL, dim = 1) {
-  # compute index names in square brackets for indexing stan parameters
-  # Args:
-  #   rownames: a vector of row names
-  #   colnames: a vector of columns 
-  #   dim: The number of dimensions of the output either 1 or 2
-  # Returns:
-  #   all index pairs of rows and cols
   if (!dim %in% c(1, 2))
     stop("dim must be 1 or 2")
   if (dim == 1) {
@@ -478,14 +432,20 @@ make_index_names <- function(rownames, colnames = NULL, dim = 1) {
   index_names
 }
 
+# save original order of the parameters in the stanfit object
+save_old_par_order <- function(x) {
+  x$fit@sim$pars_oi_old <- x$fit@sim$pars_oi
+  x$fit@sim$dims_oi_old <- x$fit@sim$dims_oi
+  x$fit@sim$fnames_oi_old <- x$fit@sim$fnames_oi
+  x
+}
+
+# perform actual renaming of Stan parameters
+# @param x a brmsfit object
+# @param change a list of lists each element allowing
+#   to rename certain parameters
+# @return a brmsfit object with updated parameter names
 do_renaming <- function(x, change) {
-  # perform actual renaming of Stan parameters
-  # Args:
-  #   change: a list of lists each element allowing
-  #     to rename certain parameters
-  #   x: An object of class brmsfit
-  # Returns:
-  #   A brmsfit object with updated parameter names
   .do_renaming <- function(x, change) {
     stopifnot(is.clist(change))
     x$fit@sim$fnames_oi[change$pos] <- change$fnames
@@ -509,10 +469,9 @@ do_renaming <- function(x, change) {
   x
 }
 
+# order parameter samples after parameter class
+# @param x brmsfit object
 reorder_pars <- function(x) {
-  # order parameter samples after parameter class
-  # Args:
-  #   x: brmsfit object
   all_classes <- unique(c(
     "b", "bs", "bsp", "bcs", "ar", "ma", "arr", "lagsar",
     "errorsar", "car", "sdcar", "sigmaLL", "sd", "cor", "df",
@@ -547,21 +506,21 @@ reorder_pars <- function(x) {
   x
 }
 
+# wrapper function to compute and store quantities in the stanfit 
+# object which were not computed / stored by Stan itself
+# @param x a brmsfit object
+# @return a brmsfit object
 compute_quantities <- function(x) {
-  # wrapper function to compute and store quantities in the stanfit 
-  # object which were not computed / stored by Stan itself
-  # Args:
-  #   x: a brmsfit object
   stopifnot(is.brmsfit(x))
   x <- compute_xi(x)
   x
 }
 
+# helper function to compute parameter xi, which is currently
+# defined in the Stan model block and thus not being stored
+# @param x a brmsfit object
+# @return a brmsfit object
 compute_xi <- function(x, ...) {
-  # helper function to compute parameter xi, which is currently
-  # defined in the Stan model block and thus not being stored
-  # Returns:
-  #   All methods return a brmsfit object
   UseMethod("compute_xi")
 }
 
@@ -599,7 +558,7 @@ compute_xi.brmsdraws <- function(x, fit, ...) {
   mu <- get_dpar(x, "mu")
   sigma <- get_dpar(x, "sigma")
   y <- matrix(x$data$Y, dim(mu)[1], dim(mu)[2], byrow = TRUE)
-  bs <- - 1 / matrixStats::rowRanges((y - mu) / sigma)
+  bs <- -1 / matrixStats::rowRanges((y - mu) / sigma)
   bs <- matrixStats::rowRanges(bs)
   temp_xi <- as.vector(as.matrix(fit, pars = temp_xi_name))
   xi <- inv_logit(temp_xi) * (bs[, 2] - bs[, 1]) + bs[, 1]
