@@ -24,11 +24,16 @@ restructure <- function(x, rstr_summary = FALSE) {
     # also added the rstan version in brms 1.5.0
     x$version <- list(brms = x$version)
   }
+  if (x$version$brms >= utils::packageVersion("brms")) {
+    return(x)
+  } 
   if (!isTRUE(attr(x, "restructured"))) {
     if (x$version$brms < "2.0.0") {
       x <- restructure_v1(x)
     }
-    x <- restructure_v2(x)
+    if (x$version$brms < "3.0.0") {
+      x <- restructure_v2(x)
+    }
   }
   stan_env <- attributes(x$fit)$.MISC
   if (rstr_summary && exists("summary", stan_env)) {
@@ -45,9 +50,10 @@ restructure <- function(x, rstr_summary = FALSE) {
 
 restructure_v2 <- function(x) {
   # restructure models fitted with brms 2.x
-  version <- x$version$brms
+  x$formula <- update_old_family(x$formula)
+  bterms <- parse_bf(x$formula)
   pars <- parnames(x)
-  bterms <- parse_bf(formula(x))
+  version <- x$version$brms
   if (version <= "2.1.1") {
     x <- do_renaming(x, change_old_bsp(pars))
   }
@@ -79,11 +85,37 @@ restructure_v2 <- function(x) {
   if (version <= "2.3.6") {
     check_old_nl_dpars(bterms)
   }
+  if (version <= "2.8.2") {
+    # argument 'sparse' is now specified within 'formula'
+    sparse <- if (grepl("sparse matrix", stancode(x))) TRUE
+    x$formula <- SW(validate_formula(
+      formula(x), data = model.frame(x), sparse = sparse
+    ))
+  }
+  if (version <= "2.8.3") {
+    x <- rescale_old_mo(x)
+  }
+  if (version <= "2.8.4") {
+    if (any(grepl("^arr(\\[|_|$)", pars))) {
+      warning2("ARR structures are no longer supported.")
+    }
+  }
+  if (version <= "2.8.5") {
+    # internal handling of special effects terms has changed
+    # this requires updating the 'terms' attribute of the data
+    x$data <- rm_attr(x$data, c("brmsframe", "terms"))
+    x$data <- update_data(x$data, bterms) 
+  }
+  if (version <= "2.8.8") {
+    if (any(grepl("^loclev(\\[|_|$)", pars))) {
+      warning2("BSTS structures are no longer supported.")
+    }
+  }
   x
 }
 
+# restructure models fitted with brms 1.x
 restructure_v1 <- function(x) {
-  # restructure models fitted with brms 1.x
   version <- x$version$brms
   if (version < "1.0.0") {
     warning2(
@@ -156,8 +188,8 @@ restructure_v1 <- function(x) {
   x
 }
 
+# convert old model formulas to brmsformula objects
 restructure_formula <- function(formula, nonlinear = NULL) {
-  # convert old model formulas to brmsformula objects
   if (is.brmsformula(formula) && is.formula(formula)) {
     # convert deprecated brmsformula objects back to formula
     class(formula) <- "formula"
@@ -181,22 +213,20 @@ restructure_formula <- function(formula, nonlinear = NULL) {
   bf(out, nl = nl)
 }
 
+# parameters to be restructured in old brmsformula objects
 old_dpars <- function() {
-  # only used when restructuring old models
   c("mu", "sigma", "shape", "nu", "phi", "kappa", "beta", "xi",
     "zi", "hu", "zoi", "coi", "disc", "bs", "ndt", "bias", 
     "quantile", "alpha", "theta")
 }
 
+# interchanges group and nlpar in names of group-level parameters
+# required for brms <= 0.10.0.9000
+# @param ranef output of tidy_ranef
+# @param pars names of all parameters in the model
+# @param dims dimension of parameters
+# @return a list whose elements can be interpreted by do_renaming
 change_old_re <- function(ranef, pars, dims) {
-  # interchanges group and nlpar in names of group-level parameters
-  # for brms <= 0.10.0.9000 only
-  # Args:
-  #   ranef: output of tidy_ranef
-  #   pars: names of all parameters in the model
-  #   dims: dimension of parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   for (id in unique(ranef$id)) {
     r <- subset2(ranef, id = id)
@@ -239,16 +269,14 @@ change_old_re <- function(ranef, pars, dims) {
   out
 }
 
+# add double underscore in group-level parameters
+# required for brms < 1.0.0
+# @note assumes that group and nlpar are correctly ordered already
+# @param ranef output of tidy_ranef
+# @param pars names of all parameters in the model
+# @param dims dimension of parameters
+# @return a list whose elements can be interpreted by do_renaming
 change_old_re2 <- function(ranef, pars, dims) {
-  # add double underscore in group-level parameters
-  # for brms < 1.0.0 only
-  # assumes that group and nlpar are correctly ordered already
-  # Args:
-  #   ranef: output of tidy_ranef
-  #   pars: names of all parameters in the model
-  #   dims: dimension of parameters
-  # Returns:
-  #   a list whose elements can be interpreted by do_renaming
   out <- list()
   for (id in unique(ranef$id)) {
     r <- subset2(ranef, id = id)
@@ -289,9 +317,9 @@ change_old_re2 <- function(ranef, pars, dims) {
   out
 }
 
+# change names of spline parameters fitted with brms <= 1.0.1
+# this became necessary after allowing smooths with multiple covariates
 change_old_sm <- function(bterms, data, pars, dims) {
-  # change names of spline parameters fitted with brms <= 1.0.1
-  # this became necessary after allowing smooths with multiple covariates
   .change_old_sm <- function(bt) {
     out <- list()
     smef <- tidy_smef(bt, data)
@@ -338,9 +366,9 @@ change_old_sm <- function(bterms, data, pars, dims) {
   out
 }
 
+# change names of monotonic effects fitted with brms <= 1.9.0
+# this became necessary after implementing monotonic interactions
 change_old_mo <- function(bterms, data, pars) {
-  # change names of monotonic effects fitted with brms <= 1.9.0
-  # this became necessary after implementing monotonic interactions
   .change_old_mo <- function(bt) {
     out <- list()
     spef <- tidy_spef(bt, data)
@@ -398,9 +426,9 @@ change_old_mo <- function(bterms, data, pars) {
   out
 }
 
+# between version 1.0 and 2.0 categorical models used
+# the internal multivariate interface
 change_old_categorical <- function(bterms, data, pars) {
-  # between version 1.0 and 2.0 categorical models used
-  # the internal multivariate interface
   stopifnot(is.brmsterms(bterms))
   if (!is_categorical(bterms$family)) {
     return(list())
@@ -424,17 +452,16 @@ change_old_categorical <- function(bterms, data, pars) {
   list(nlist(pos, fnames = new_pars[pos]))
 }
 
+# as of brms 2.2 'mo' and 'me' terms are handled together
 change_old_bsp <- function(pars) {
-  # as of brms 2.2 'mo' and 'me' terms are handled together
   pos <- grepl("^(bmo|bme)_", pars)
   if (!any(pos)) return(list()) 
   fnames <- gsub("^(bmo|bme)_", "bsp_", pars[pos])
   list(nlist(pos, fnames))
 }
 
+# prepare for renaming of parameters in old models
 change_simple <- function(oldname, fnames, pars, dims, pnames = fnames) {
-  # helper function for very simple renaming
-  # only used in renaming of old models
   pos <- grepl(paste0("^", oldname), pars)
   if (any(pos)) {
     out <- nlist(pos, oldname, pnames, fnames, dims = dims[[oldname]])
@@ -445,6 +472,114 @@ change_simple <- function(oldname, fnames, pars, dims, pnames = fnames) {
   out
 }
 
+# rescale old 'b' coefficients of monotonic effects 
+# to represent average instead of total differences
+rescale_old_mo <- function(x, ...) {
+  UseMethod("rescale_old_mo")
+}
+
+#' @export
+rescale_old_mo.brmsfit <- function(x, ...) {
+  bterms <- parse_bf(x$formula)
+  rescale_old_mo(bterms, fit = x, ...)
+}
+
+#' @export
+rescale_old_mo.mvbrmsterms <- function(x, fit, ...) {
+  for (resp in x$responses) {
+    fit <- rescale_old_mo(x$terms[[resp]], fit = fit, ...)
+  }
+  fit
+}
+
+#' @export
+rescale_old_mo.brmsterms <- function(x, fit, ...) {
+  for (dp in names(x$dpars)) {
+    fit <- rescale_old_mo(x$dpars[[dp]], fit = fit, ...)
+  }
+  for (nlp in names(x$nlpars)) {
+    fit <- rescale_old_mo(x$nlpars[[nlp]], fit = fit, ...)
+  }
+  fit
+}
+
+#' @export
+rescale_old_mo.btnl <- function(x, fit, ...) {
+  fit
+}
+
+#' @export
+rescale_old_mo.btl <- function(x, fit, ...) {
+  spef <- tidy_spef(x, fit$data)
+  has_mo <- lengths(spef$Imo) > 0L
+  if (!any(has_mo)) {
+    return(fit)
+  }
+  warning2(
+    "The parameterization of monotonic effects has changed in brms 2.8.4 ",
+    "so that corresponding 'b' coefficients now represent average instead ",
+    "of total differences between categories. See vignette('brms_monotonic') ", 
+    "for more details. Parameters of old models are adjusted automatically."
+  )
+  p <- combine_prefix(x)
+  all_pars <- parnames(fit)
+  chains <- fit$fit@sim$chains
+  for (i in which(has_mo)) {
+    bsp_par <- paste0("bsp", p, "_", spef$coef[i])
+    simo_regex <- paste0(spef$coef[i], seq_along(spef$Imo[[i]]))
+    simo_regex <- paste0("simo", p, "_", simo_regex, "[")
+    simo_regex <- paste0("^", escape_all(simo_regex))
+    # scaling factor by which to divide the old 'b' coefficients
+    D <- prod(ulapply(simo_regex, function(r) sum(grepl(r, all_pars))))
+    for (j in seq_len(chains)) {
+      fit$fit@sim$samples[[j]][[bsp_par]] <- 
+        fit$fit@sim$samples[[j]][[bsp_par]] / D
+    }
+  }
+  fit
+}
+
+# update old families to work with the latest brms version
+update_old_family <- function(x, ...) {
+  UseMethod("update_old_family")
+}
+
+#' @export
+update_old_family.default <- function(x, ...) {
+  check_family(x)
+}
+
+#' @export
+update_old_family.brmsfamily <- function(x, ...) {
+  # new specials may have been added in new brms versions
+  family_info <- get(paste0(".family_", x$family))()
+  x$specials <- family_info$specials
+  x
+}
+
+#' @export
+update_old_family.customfamily <- function(x, ...) {
+  x
+}
+
+#' @export
+update_old_family.mixfamily <- function(x, ...) {
+  x$mix <- lapply(x$mix, update_old_family, ...)
+  x
+}
+
+#' @export
+update_old_family.brmsformula <- function(x, ...) {
+  x$family <- update_old_family(x$family, ...)
+  x
+}
+
+#' @export
+update_old_family.mvbrmsformula <- function(x, ...) {
+  x$forms <- lapply(x$forms, update_old_family, ...)
+  x
+}
+
 stop_parameterization_changed <- function(family, version) {
   stop2(
     "The parameterization of '", family, "' models has changed in brms ",
