@@ -117,7 +117,7 @@ test_that("log_lik for ARMA covariance models runs without errors", {
   draws$ac <- list(
     ar = matrix(rbeta(ns, 0.5, 0.5), ncol = 1),
     ma = matrix(rbeta(ns, 0.2, 1), ncol = 1),
-    begin_tg = 2, nobs_tg = 4
+    begin_tg = 2, end_tg = 5
   )
   draws$data <- list(Y = rnorm(nobs), se = rgamma(ns, 10))
 
@@ -370,9 +370,10 @@ test_that("log_lik for ordinal models runs without erros", {
   ncat <- 4
   draws <- structure(list(nsamples = ns, nobs = nobs), class = "brmsdraws")
   draws$dpars <- list(
-    mu = array(rnorm(ns*nobs), dim = c(ns, nobs, ncat)),
+    mu = array(rnorm(ns * nobs), dim = c(ns, nobs)),
     disc = rexp(ns)
   )
+  draws$thres <- array(0, dim = c(ns, ncat - 1))
   draws$data <- list(Y = rep(1:ncat, 2), ncat = ncat)
   draws$family$link <- "logit"
   
